      SUBROUTINE ATMFIL ( LUNATM, NAMATM, FIRST, GOTPRE, GOTTEM, 
     &                    GOTGAS, FAIL, ERRMSG )
C
C VERSION
C     01JAN15 AD Allow for TVib gradients
C                Change arguments for FILPRF, ATMPSI, PRFGRA
C     16DEC14 AD Set FIRST to .FALSE. locally
C                Restructured and simplified
C                Add ATMLAY
C     07AUG13 AD Remove redundant ATMNTE from EXTERNAL list
C     09JAN08 AD Add ATMPSI to extract PSI angle
C                    Use CHKGAS to test for molecule name
C                    Remove CFCGAS and move ATMISO
C     06JAN04 AD Check that PSI value is less than max absolute size
C     20JUL02 AD Insist that all profiles have PSI values with GRA flag
C     06MAR02 AD Correction: only check ISOGAS if line species
C     16FEB02 AD Add ATMISO.
C                    Increase LABEL from C*8 to C*20
C     06JUN01 AD Add ATMCHK
C     01SEP96 AD Version 1.
C
C DESCRIPTION
C     Read file containing atmospheric profiles.
C     Called by INPATM for each file listed in *ATM section of driver table.
C     The logic is
C       If First File then
C         Read No.levels and set NATM
C         Use first profile to set heights in HGTATM 
C         Copy any other profiles direct to *ATM arrays
C       Else 
C         Read No.levels and set NLEV
C         Use first profile to set heights in HGTLEV
C         Read any other profiles into PRFLEV
C         Interpolate any other profiles from PRFLEV to *ATM with HGTLEV,HGTATM
C       End if
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER       LUNATM    !  I  LUN for Atmospheric profiles 
      CHARACTER*(*) NAMATM    !  I  Name of file 
      LOGICAL       FIRST     ! I/O TRUE=use these profiles to define grid
      LOGICAL       GOTTEM    ! I/O Set TRUE if Temperature profile read in
      LOGICAL       GOTPRE    ! I/O Set TRUE if Pressure profile read in
      LOGICAL       GOTGAS(*) ! I/O Set TRUE if vmr profile read in
      LOGICAL       FAIL      !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG    !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  ATMCHK ! Check atmospheric profiles are reasonable.
     &, ATMISO ! Create separate isotope profiles if specified in .atm file
     &, ATMLAY ! Subdivide atmospheric profile layers.
     &, ATMPSI ! Extract Psi angle from brackets following .atm filename
     &, CHKGAS ! Check if GAS is valid molecule and get Hitran/RFM index
     &, FILPRF ! Read atmospheric profile from file into /ATMCOM/.
     &, IDXNTE ! Index of Vib.Temp data in /NTECOM/ arrays
     &, NAMGAS ! Return molecular name associated with /GASCOM/ index
     &, NXTLAB ! Load next label from RFM .atm file
     &, OPNFIL ! Open ASCII input file, and skip/Log any initial comments.
     &, PRFGRA ! Copy single profile from /ATMCOM/ to /GRACOM/.
     &, RFMLOG ! Write text message to RFM log file.
      INTEGER IDXNTE
      CHARACTER*7 NAMGAS
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C COMMON VARIABLES
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'gascom.inc' ! Molecular and isotope data
C
C LOCAL VARIABLES
      INTEGER      IDUM    ! Dummy variable
      INTEGER      IDXVIB  ! Encoded vibrational level information
      INTEGER      IGAS    ! Gas profile counter (1:NGAS)
      INTEGER      IGQ     ! Global Quantum Index (currently dummy variable)
      INTEGER      IGRA    ! Index of horizontal location (or 0)
      INTEGER      ILEV    ! Profile point counter
      INTEGER      IMOL    ! RFM/HITRAN index of recognised molecule, else 0
      INTEGER      INTE    ! Index of Vib.Temp profile
      INTEGER      IOS     ! Saved value of IOSTAT for error message
      INTEGER      ISO     ! Isotope# (0=no isotope specified)
      INTEGER      NLEV    ! No. levels in supplied profiles
      REAL         RDUMMY  ! Dummy real for read
      LOGICAL      ANYUSE  ! T = at least one profile used from file
      CHARACTER*20 LABEL   ! Label identifying profile
      CHARACTER*7  GAS     ! Possible Molecule name extracted from LABEL(2:)
      CHARACTER*80 MESSGE  ! Text message for LOG file
      CHARACTER*80 RECORD  ! Line of Driver file or External ATM file
C
C EXECUTABLE CODE -------------------------------------------------------------
C
C Check for any horizontal angle PSI following .atm filename
      CALL ATMPSI ( NAMATM, IGRA, FAIL, ERRMSG )  ! Returns IGRA=0 if not GRAFLG
      IF ( FAIL ) RETURN
C
      CALL OPNFIL ( LUNATM, NAMATM, FAIL, ERRMSG ) 
      IF ( FAIL ) RETURN
      ANYUSE = FIRST
C
C 1st non-comment record in file must be no. levels (integer)
      READ ( LUNATM, *, IOSTAT=IOS, ERR=900 ) NLEV
C
C First profile label should be '*hgt'
      CALL NXTLAB ( LUNATM, LABEL, RECORD, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      IF ( LABEL .EQ. '*hgt' ) THEN
        CALL RFMLOG ( '    '//RECORD, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
        CALL FILPRF ( LUNATM, FIRST, NLEV, 'Z', 0, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
        IF ( LAYFLG .AND. FIRST ) THEN
          CALL ATMLAY ( FAIL, ERRMSG )  ! add extra altitude levels
          FIRST = .FALSE.   ! rest of .atm file profiles will be interpolated
        END IF
      ELSE
        FAIL = .TRUE.
        ERRMSG = 'F-ATMFIL: 1st profile wasn''t *hgt, it was:'//LABEL
        RETURN
      END IF
C
C Read remaining profiles in file
      CALL NXTLAB ( LUNATM, LABEL, RECORD, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      DO WHILE ( LABEL .NE. '*end' )
C Check for pressure profile
        IF ( LABEL .EQ. '*pre' ) THEN                 
          CALL RFMLOG ( '    '//RECORD, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          CALL FILPRF ( LUNATM, FIRST, NLEV, 'P', 0, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          IF ( GRAFLG ) CALL PRFGRA ( IGRA, 'P', 0 ) 
          IF ( GOTPRE ) THEN                         ! Overwritten previous
            MESSGE = 'W-ATMFIL: Pressure profile superseded'
            CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
          END IF
          GOTPRE = .TRUE.                               ! Flag as loaded
          ANYUSE = .TRUE.
C Check for temperature profile
        ELSE IF ( LABEL .EQ. '*tem' ) THEN           
          CALL RFMLOG ( '    '//RECORD, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          CALL FILPRF ( LUNATM, FIRST, NLEV, 'T', 0, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          IF ( GRAFLG ) CALL PRFGRA ( IGRA, 'T', 0 )
          IF ( GOTTEM ) THEN                        ! Overwritten previous 
            MESSGE = 'W-ATMFIL: Temperature profile superseded'
            CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
          END IF
          GOTTEM = .TRUE.                                  ! Flag as loaded
          ANYUSE = .TRUE.
C Check for recognised absorbing molecule from those loaded in *GAS section
        ELSE                                              
          CALL CHKGAS ( LABEL(2:), GAS, IMOL, ISO, IGQ, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          IF ( IMOL .EQ. 0 ) THEN                    ! Not a recognised molecule
            READ ( LUNATM, *, IOSTAT=IOS, ERR=900 )  ! Read past profile data
     &            ( RDUMMY, ILEV = 1, NLEV)
          ELSE IF ( IGSMOL(IMOL) .EQ. 0 ) THEN       ! Not using this molecule
            READ ( LUNATM, *, IOSTAT=IOS, ERR=900 )  ! Read past profile data
     &            ( RDUMMY, ILEV = 1, NLEV)
          ELSE IF ( IGQ .NE. 0 .AND. NTEFLG ) THEN   ! VT Profile
            CALL RFMLOG ( '    '//RECORD, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
            IDXVIB = IMOL * 1000000 + ISO * 1000 + IGQ
            CALL FILPRF ( LUNATM, FIRST, NLEV, 'N', IDXVIB, FAIL,ERRMSG)
            IF ( FAIL ) RETURN
            IF ( GRAFLG ) THEN
              INTE = IDXNTE ( IMOL, ISO, IGQ )
              CALL PRFGRA ( IGRA, 'N', INTE )
            END IF
            ANYUSE = .TRUE.
          ELSE                                       ! Use this VMR profile
            IGAS = IGSMOL(IMOL)
C Establish separate atmospheric profile for this isotopomer if required
            IF ( ISO .GT. 0 ) THEN                     ! Isotopic profile
              CALL ATMISO ( IMOL, ISO, FAIL, ERRMSG )  ! if new, increase NGAS
              IF ( FAIL ) RETURN
              IGAS = ISOGAS(ISO,IGAS)                  ! Isotopic profile
            END IF
            CALL RFMLOG ( '    '//RECORD, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
            CALL FILPRF ( LUNATM, FIRST, NLEV, 'V', IGAS, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
            IF ( GRAFLG ) CALL PRFGRA ( IGRA, 'V', IGAS )
            IF ( GOTGAS(IGAS) ) THEN                              
              MESSGE =
     &          'W-ATMFIL: '//NAMGAS(IGAS,IDUM)//' profile superseded'
              CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
              IF ( FAIL ) RETURN
            END IF
            GOTGAS(IGAS) = .TRUE.                               
            ANYUSE = .TRUE.
          END IF              
        END IF               
        CALL NXTLAB ( LUNATM, LABEL, RECORD, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
      END DO
C
      IF ( .NOT. ANYUSE ) THEN
        MESSGE = 'W-ATMFIL: No useful profiles found in file'
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      END IF
      CLOSE ( LUNATM, IOSTAT=IOS, ERR=900 )
      FAIL = .FALSE.
C
C Check currently loaded profiles are reasonable
      CALL ATMCHK ( GOTTEM, GOTPRE, GOTGAS, FAIL, ERRMSG ) 
      IF ( FAIL ) RETURN
C
C Normal exit
      FIRST = .FALSE.
      RETURN
C
  900 WRITE ( ERRMSG, '(A,I11)' )
     &  'F-ATMFIL: I/O failure reading Atm.Profile file. IOSTAT=', IOS
      FAIL = .TRUE.
C
      END
