      SUBROUTINE CIAFIL ( LUNCIA, NAMCIA, FAIL, ERRMSG )
C
C VERSION
C     25SEP15 AD Correction to log message info
C     12JUL15 AD Original. 
C
C DESCRIPTION
C     Read Collision-Induced Abs. data file
C     Called by INPCIA for each file specified in *CIA section of driver table.
C     Treats .cia files as arbitrary concatenations of datasets.
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER       LUNCIA  !  I  LUN for reading CIA Files
      CHARACTER*(*) NAMCIA  !  I  Name of CIA file
      LOGICAL       FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG  !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  OPNFIL ! Open ASCII input file, and skip/Log any initial comments.
     &, REACIA ! Read dataset from .cia file
     &, RFMLOG ! Write text message to RFM log file.
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C LOCAL CONSTANTS
      REAL TEMTOL          ! Tolerance [K] for matching dataset temperatures
        PARAMETER ( TEMTOL = 0.1 ) 
      DOUBLE PRECISION WNOTOL  ! Tolerance [cm-1] for matching spectral ranges
        PARAMETER ( WNOTOL = 1.0D0 )
C
C COMMON VARIABLES
      INCLUDE 'gascom.inc' ! Molecular and isotope data
      INCLUDE 'ciacom.inc' ! Coll.Ind.Absorption data 
      INCLUDE 'spccom.inc' ! Spectral Range & Resolution data
C
C LOCAL VARIABLES
      INTEGER   ID1,ID2    ! HITRAN/RFM indices of collision molecules
      INTEGER       IOS    ! Saved value of IOSTAT for error message
      INTEGER       ICIA   ! Indices of CIA data sets
      INTEGER       LCIA   ! Saved value of NCIA at start
      INTEGER       MAXPT  ! Max. no of new data points that can be read
      INTEGER       NCAP   ! No. data points stored so far.
      INTEGER       NPT    ! No. data points actually read from file
      INTEGER       NPTREQ ! No. data points required to read all data
      INTEGER       NTEM   ! No. temperature tabulations for each spec.range
      LOGICAL       EOF    ! T=reached end-of-file, F=not yet EOF
      LOGICAL       NEWSPC ! T=new spectral range within file data
      REAL          TEM    ! Temperature [K] associated with CIA dataset
      REAL          TMAX,TMIN ! Max/Min Temp [K] for each spectral range
      CHARACTER*80  LOGMSG ! Summary message for Log file
      CHARACTER*80  WRNMSG ! Warning message for Log file
      DOUBLE PRECISION WN1,WN2 ! Min/Max Wno [cm-1] read for dataset
      DOUBLE PRECISION WN1SAV, WN2SAV ! Saved values of WN1, WN2
C
C DATA STATEMENTS
      DATA NCAP / 0 /
      SAVE NCAP
C
C EXECUTABLE CODE --------------------------------------------------------------
C
      LCIA = NCIA
C Open file and print first record to log file
      CALL OPNFIL ( LUNCIA, NAMCIA, FAIL, ERRMSG ) 
      IF ( FAIL ) RETURN
C
      MAXPT = MAXCAP
C WMNSPC,WMXSPC from spccom.inc
      CALL REACIA ( LUNCIA, WMNSPC, WMXSPC, MAXPT, EOF,
     &              ID1, ID2, TEM, NPT, NPTREQ, 
     &              WNOCIA(NCAP+1), ABSCIA(NCAP+1), FAIL, ERRMSG )   
      IF ( FAIL ) RETURN 
      DO WHILE ( .NOT. EOF ) 
        WN1 = WNOCIA(NCAP+1)
        WN2 = WNOCIA(NCAP+NPT)
        IF ( IGSMOL(ID1) .GT. 0 .AND. IGSMOL(ID2) .GT. 0 ) THEN
C Check for any conflict with data already loaded
          DO ICIA = 1, NCIA
            IF ( ID1 .EQ. ID1CIA(ICIA) .AND. 
     &           ID2 .EQ. ID2CIA(ICIA) .AND.
     &           ABS ( TEM - TEMCIA(ICIA) ) .LE. TEMTOL .AND.
     &           WN1 .LT. WN2CIA(ICIA) .AND.
     &           WN2 .GT. WN1CIA(ICIA) ) THEN
              FAIL = .TRUE.
              ERRMSG = 'F-CIAFIL: Repeated CIA dataset'
              RETURN
            END IF
          END DO
C Check enough array space to accommodate data
          IF ( NCIA .GE. MAXCIA ) THEN
            FAIL = .TRUE.
            WRITE ( ERRMSG, '(A,I3,A)' )  
     &        'F-CIAFIL: No. CIA datasets > MAXCIA=', MAXCIA,  
     &        ' in rfmsiz.inc'
            RETURN
          END IF
          IF ( NPTREQ .GT. NPT ) THEN
            WRITE ( ERRMSG, '(A,I7,A)' )  
     &        'F-CIAFIL: No. CIA data points > MAXCAP=', MAXCAP,  
     &        ' in rfmsiz.inc'
            FAIL = .TRUE.
            RETURN
          END IF
C Load data into /CIACOM/ in ciacom.inc
          NCIA = NCIA + 1
          IOFCIA(NCIA) = NCAP
          NPTCIA(NCIA) = NPT
          ID1CIA(NCIA) = ID1
          ID2CIA(NCIA) = ID2
          WN1CIA(NCIA) = WN1
          WN2CIA(NCIA) = WN2
          TEMCIA(NCIA) = TEM      
          NCAP = NCAP + NPT
C
          MAXPT = MAXPT - NPT
        END IF
C Read next dataset in file
        CALL REACIA ( LUNCIA, WMNSPC, WMXSPC, MAXPT, EOF,
     &                ID1, ID2, TEM, NPT, NPTREQ, 
     &                WNOCIA(NCAP+1), ABSCIA(NCAP+1), FAIL, ERRMSG )   
        IF ( FAIL ) RETURN
      END DO
C
      IF ( NCIA .EQ. LCIA ) THEN
        WRNMSG = 'W-CIAFIL: File ignored - contains no required data'
        CALL RFMLOG ( WRNMSG, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      END IF
C Print summary of used file contents to log file
      WN1SAV = WN1CIA(LCIA+1)
      WN2SAV = WN2CIA(LCIA+1)
      TMIN = TEMCIA(LCIA+1)
      TMAX = TEMCIA(LCIA+1)
      NTEM = 1
      DO ICIA = LCIA+2, NCIA
        NEWSPC = ABS ( WN1CIA(ICIA) - WN1SAV ) .GT. WNOTOL .OR.
     &           ABS ( WN2CIA(ICIA) - WN2SAV ) .GT. WNOTOL
        IF ( NEWSPC ) THEN
          IF ( NTEM .EQ. 1 ) THEN
            WRITE ( LOGMSG, '(A,F11.4,A,F11.4,A,I4,A,F6.1,A)' )
     &        'I-CIAFIL:', WN1SAV, ' -', WN2SAV, ' cm-1 read', NTEM,
     &        ' Temp at', TMIN, ' K'
          ELSE
            WRITE ( LOGMSG, '(A,F11.4,A,F11.4,A,I4,A,F6.1,A,F6.1,A)' )
     &        'I-CIAFIL:', WN1SAV, ' -', WN2SAV, ' cm-1 read', NTEM,
     &        ' Temps from', TMIN, ' -', TMAX, ' K'
          END IF
          CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          WN1SAV = WN1CIA(ICIA)
          WN2SAV = WN2CIA(ICIA)
          TMIN = TEMCIA(ICIA)
          TMAX = TEMCIA(ICIA)
        ELSE          
          NTEM = NTEM + 1
          TMIN = MIN ( TMIN, TEMCIA(ICIA) )
          TMAX = MAX ( TMAX, TEMCIA(ICIA) ) 
        END IF
      END DO
      IF ( NTEM .EQ. 1 ) THEN
        WRITE ( LOGMSG, '(A,F11.4,A,F11.4,A,I4,A,F6.1,A)' )
     &   'I-CIAFIL:', WN1SAV, ' -', WN2SAV, ' cm-1 read', NTEM,
     &   ' Temp at', TMIN, ' K'
      ELSE
        WRITE ( LOGMSG, '(A,F11.4,A,F11.4,A,I4,A,F6.1,A,F6.1,A)' )
     &    'I-CIAFIL:', WN1SAV, ' -', WN2SAV, ' cm-1 read', NTEM,
     &    ' Temps from', TMIN, ' -', TMAX, ' K'
      END IF
      CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
      CLOSE ( LUNCIA, IOSTAT=IOS, ERR=900 )
C
C Normal exit
      RETURN
C
C Exit with fatal I/O error
  900 FAIL = .TRUE.
      WRITE ( ERRMSG, '(A,I11)' ) 
     &  'F-CIAFIL: I/O failure on X/S file. IOSTAT=', IOS

      END

