      SUBROUTINE GASCTM ( ANYQAL, LQS, QALSTR, FAIL, ERRMSG )
C
C VERSION
C     25SEP15 AD Allow for '(cia)' qualifier
C     15JUL15 AD Add SUB and NOSUB qualifier options
C                Remove CTMFLG argument, use flgcom.inc instead
C     20JAN15 AD Tidied up. Slight reordering of NOCTM case.
C     01DEC97 AD Original
C
C DESCRIPTION
C     Initialise for Continuum data
C     Called by GASCHK for any gas with a qualifier string.
C     Also checks for (ctm) or (noctm) qualifiers appended to gas name
C     If (ctm) qualifier is present (=continuum only)
C       Check CTM flag is set (else fatal error)
C       Check CTMGAS flag is set for this molecule (else fatal error)
C       Set SHPGAS=SHPCTM (flag for continuum only shape)
C       Check no other qualifiers present (else fatal error)
C     Else if (noctm) qualifier present (=exclude continuum)
C       Check CTM flag is set (else warning)
C       Check CTMGAS is set for this molecule (else warning)
C       Set CTMGAS=.FALSE.
C       Remove this qualifier from QALSTR
C       Check if other qualifiers present, else set ANYQAL=.FALSE.
C     End if
C
      IMPLICIT NONE
C
C ARGUMENTS
      LOGICAL       ANYQAL  ! I/O (Further) qualifiers present 
      INTEGER       LQS     ! I/O Length of QALSTR (1:LQS)
      CHARACTER*(*) QALSTR  ! I/O String containing list of qualifiers
      LOGICAL       FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG  !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  CMPSTR ! Remove substring from string.
     &, LOCASE ! Convert text string to lower case.
     &, RFMLOG ! Write text message to RFM log file.
C
C GLOBAL CONSTANTS
      INCLUDE 'idxcon.inc' ! HITRAN molecular indices
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
      INCLUDE 'shpcon.inc' ! Line-shape codes
C
C COMMON VARIABLES
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'gascom.inc' ! Molecular and isotope data
C
C LOCAL VARIABLES
      INTEGER      IPT     ! Pointer to '(' element of current qualifier 
      CHARACTER*80 LOGMSG  ! Message sent to RFM log file
C
C EXECUTABLE CODE --------------------------------------------------------------
C
      FAIL = .FALSE.
C
C Initialise continuum flags for those species with continuum data
      IF ( IDXGAS(NGAS) .EQ. IDXH2O .OR.
     &     IDXGAS(NGAS) .EQ. IDXCO2 .OR.
     &     IDXGAS(NGAS) .EQ. IDXO2  .OR.
     &     IDXGAS(NGAS) .EQ. IDXN2       )  THEN
        CTMGAS(NGAS) = CTMFLG
      ELSE
        CTMGAS(NGAS) = .FALSE.
      END IF
C For H2O, default is to subtract 25cm-1 baseline when used in combination
C with continuum, or when generating LUTs
      IF ( IDXGAS(NGAS) .EQ. IDXH2O ) SUBH2O = CTMFLG .OR. TABFLG
      IF ( .NOT. ANYQAL ) RETURN
C
      CALL LOCASE ( QALSTR, QALSTR )     
C
C First check for presence of (ctm) qualifier in QALSTR
      IPT = INDEX ( QALSTR, '(ctm)' )
      IF ( IPT .NE. 0 ) THEN
        IF ( .NOT. CTMFLG ) THEN
          FAIL = .TRUE.
          ERRMSG = 'F-GASCTM: (ctm) qualifier set for gas='//
     &      CODGAS(NGAS)//' but CTM flag not enabled'
          RETURN
        ELSE IF ( .NOT. CTMGAS(NGAS) ) THEN
          FAIL = .TRUE. 
          ERRMSG = 'F-GASCTM: No continuum data available for gas='
     &      //CODGAS(NGAS)
          RETURN
        END IF
        SHPGAS(NGAS) = SHPCTM
        LOGMSG = 'I-GASCTM: setting gas='//CODGAS(NGAS)//
     &        ' to continuum-only calculation'
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
C
        CALL CMPSTR ( QALSTR, IPT, IPT+4, LQS )    ! remove '(ctm)' 
        ANYQAL = LQS .GT. 0
        IF ( ANYQAL ) THEN
          FAIL = .TRUE.
          ERRMSG = 'F-GASCTM: No additional qualifiers allowed'//
     &             ' with (CTM)'
        END IF
        RETURN                     ! exit for (ctm) only qualifier
      END IF
C
C Then check for presence of (cia) qualifier in QALSTR
      IPT = INDEX ( QALSTR, '(cia)' )
      IF ( IPT .NE. 0 ) THEN
        IF ( .NOT. CIAFLG ) THEN
          FAIL = .TRUE.
          ERRMSG = 'F-GASCTM: (cia) qualifier set for gas='//
     &      CODGAS(NGAS)//' but CIA flag not enabled'
          RETURN
        END IF
        SHPGAS(NGAS) = SHPCTM
        LOGMSG = 'I-GASCTM: setting gas='//CODGAS(NGAS)//
     &        ' to CIA-only calculation'
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
C
        CALL CMPSTR ( QALSTR, IPT, IPT+4, LQS )    ! remove '(cia)' 
        ANYQAL = LQS .GT. 0
        IF ( ANYQAL ) THEN
          FAIL = .TRUE.
          ERRMSG = 'F-GASCTM: No additional qualifiers allowed'//
     &             ' with (CIA)'
        END IF
        RETURN                     ! exit for (cia) only qualifier
      END IF
C
C Check for (noctm) qualifier
      IPT = INDEX ( QALSTR, '(noctm)' ) 
      IF ( IPT .NE. 0 ) THEN
        CALL CMPSTR ( QALSTR, IPT, IPT+6, LQS )  ! remove '(noctm)' from QALSTR
        ANYQAL = LQS .GT. 0
        IF ( .NOT. CTMFLG ) THEN
          LOGMSG = 'W-GASCTM: CTM Flag not set so (noctm) qualifier'//
     &      ' for gas='//CODGAS(NGAS)//' is redundant'
        ELSE IF ( .NOT. CTMGAS(NGAS) ) THEN        
          LOGMSG = 'W-GASCTM: No continuum data for gas='//
     &      CODGAS(NGAS)//' so (noctm) qualifier is redundant'
        ELSE
          CTMGAS(NGAS) = .FALSE.
          IF ( IDXGAS(NGAS) .EQ. IDXH2O ) SUBH2O = TABFLG
          LOGMSG = 'I-GASCTM: Excluding continuum for gas='
     &      //CODGAS(NGAS)
        END IF
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
C If no continuum for H2O, subtraction of 25cm-1 offset depends only on TAB flag
        IF ( IDXGAS(NGAS) .EQ. IDXH2O ) SUBH2O = TABFLG
      END IF
C
      IF ( IDXGAS(NGAS) .EQ. IDXH2O ) THEN
        IF ( SUBH2O ) THEN
          LOGMSG = 'I-GASCTM: Default setting: '//
     &             'subtract 25cm-1 offset from H2O lines'
        ELSE
          LOGMSG = 'I-GASCTM: Default setting: '//
     &             'do not subtract 25cm-1 offset from H2O lines'
        END IF
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      END IF
C
C Check for (sub) qualifier
      IPT = INDEX ( QALSTR, '(sub)' )
      IF ( IPT .NE. 0 ) THEN
        IF ( IDXGAS(NGAS) .NE. IDXH2O ) THEN
          ERRMSG = 'F-GASCTM: (SUB) qualifier only applicable to H2O'
          FAIL = .TRUE.
          RETURN
        ELSE IF ( INDEX ( QALSTR, '(nosub)' ) .GT. 0 ) THEN
          ERRMSG = 'F-GASCTM: both (SUB) and (NOSUB) qualifiers' //
     &             ' not allowed'
          FAIL = .TRUE.
          RETURN
        END IF
        CALL CMPSTR ( QALSTR, IPT, IPT+4, LQS )  ! remove '(sub)' from QALSTR
        ANYQAL = LQS .GT. 0
        IF ( SUBH2O ) THEN
          LOGMSG = 'W-GASCTM: (SUB) already set by default for H2O'
        ELSE
          SUBH2O = .TRUE.
          LOGMSG = 'I-GASCTM: (SUB): change to subtract 25cm-1 '//
     &             'offset from H2O lines'
        END IF
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        RETURN           ! Normal exit with (sub) qualifier
      END IF
C
C Check for (nosub) qualifier
      IPT = INDEX ( QALSTR, '(nosub)' )
      IF ( IPT .NE. 0 ) THEN
        IF ( IDXGAS(NGAS) .NE. IDXH2O ) THEN
          ERRMSG = 'F-GASCTM: (NOSUB) qualifier only applicable to H2O'
          FAIL = .TRUE.
          RETURN
        END IF
        CALL CMPSTR ( QALSTR, IPT, IPT+6, LQS ) 
        ANYQAL = LQS .GT. 0
        IF ( .NOT. SUBH2O ) THEN
          LOGMSG = 'W-GASCTM: (NOSUB) already set by default for H2O'
        ELSE
          SUBH2O = .FALSE.
          LOGMSG = 'I-GASCTM: (NOSUB): change to not subtract '//
     &             '25cm-1 offset from H2O lines'
        END IF
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
      END IF
C
      END 
