      SUBROUTINE GASQAL ( QALSTR, IGAS, CODGAS, NISGAS, FAIL, ERRMSG )
C
C VERSION
C     02SEP15 AD Correction: allow ILS,IUS=0, equiv. to '*'
C     05AUG15 AD Allow -ve Isotope values to exclude particular isotopes
C     02APR15 AD Add CODGAS and NISGAS as arguments, remove gascom.inc
C                Use REAQAL to decode qualifier values
C     31JAN04 AD Correct format statement for error message
C     22JAN98 AD Split concatenation so FORTRAN to C converter can cope
C     02OCT97 AD Allow band-isotope combinations.
C     22JUL97 AD Original
C
C DESCRIPTION
C     Check & load isotope/band qualifiers for each molecule 
C     Called by GASCHK for each molecule with appended qualifier string.
C     The layout of qualifiers is, eg for CO2,
C       CO2(3:4)(1:2)(2)(1)(2:3)(3)(1:3)(2:4)(4)(5)
C     which will (a) select bands (3:4), (1:2) for all isotopes
C                (b) select band (2:3) for isotopes 1,2
C                (c) select bands (1:3), (2:3) for isotope 3
C                (d) select all bands for isotopes 4,5
C     Implicitly this is CO2(*)(user-qualifiers)(*:*), ie starts with all
C     isotopes selected, ends with all bands selected. Decoded as pairs of 
C     (isotopes)x(applicable bands)
C
C     The structure of this subroutine is:
C       Initialise ISOLST with ALL isotopes for gas
C       For each Qualifier ...
C         If Qualifier is a Band ...
C           Set band for all isotopes currently stored in ISOLST
C         Else Qualifier is an Isotope
C           If previous Qualifier was a band then erase current ISOLST
C           Add Isotope to ISOLST
C         End if
C       End loop over Qualifiers
C       If last Qualifier was an Isotope then set all bands for current ISOLST
C
      IMPLICIT NONE
C
C ARGUMENTS
      CHARACTER*(*) QALSTR !  I  String containing list of qualifiers
      INTEGER       IGAS   !  I  Index of molecule in /GASCOM/ arrays
      CHARACTER*7   CODGAS !  I  Name of molecule
      INTEGER       NISGAS !  I  Total number of defined isotopes for molecule
      LOGICAL       FAIL   !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  REAQAL ! Decode Qualifier strings '(I1)' or '(I1:I2)'
     &, RFMLOG ! Write text message to RFM log file.
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C COMMON VARIABLES
      INCLUDE 'qalcom.inc' ! List of band/isotope qualifiers for line data.
C
C LOCAL VARIABLES
      INTEGER      IBND    ! Selected band counter
      INTEGER      IISO    ! Isotope counter
      INTEGER      ILS     ! Selected lower state Global Quantum number
      INTEGER      IPT     ! Pointer to '(' element of current qualifier 
      INTEGER      IQAL    ! Qualifier counter
      INTEGER      ISO     ! Isotope counter/index
      INTEGER      ISOLST(MAXISO) ! List of isotopes using current band
      INTEGER      IUS     ! Selected upper state Global Quantum number
      INTEGER      JISO    ! Secondary isotope counter
      INTEGER      JPT     ! Pointer to ')' element of current qualifier 
      INTEGER      LPT     ! Pointer to last element of QALSTR
      INTEGER      MPT     ! Pointer to limit length of string written to ERRMSG
      INTEGER      NISO    ! No.of isotopes in ISOLST
      LOGICAL      LASBND  ! T=last qualifier read was a band (F=an isotope)
      LOGICAL      ONEQAL  ! T=single qualifier, F=pair of qualifiers
      CHARACTER*30 ERRSTR  ! Substring for error messages
      CHARACTER*80 LOGMSG  ! Message sent to RFM log file
C
C EXECUTABLE CODE --------------------------------------------------------------
C
      LPT = LEN ( QALSTR )
C
C Initialise for each new gas
      USEQAL(IGAS) = .TRUE.
      IQAL = 0
      IPT = 1
      JPT = 0
      LASBND = .TRUE. 
      NISO = NISGAS
      DO ISO = 1, NISO
        ISOLST(ISO) = ISO
        NBND(ISO,IGAS) = 0
      END DO
      LOGMSG = 'I-GASQAL: Gas=' // CODGAS //
     &      ' List of selected isotopes and/or bands ... '
      CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
C
C Repeat until all qualifiers decoded from QALSTR
      DO WHILE ( JPT .LT. LPT )
        IQAL = IQAL + 1
C
C Set up substring for any error messages applicable to this gas/qualifier
        WRITE ( ERRSTR, '(A,I2,A)' )
     &    'F-GASQAL: Gas='//CODGAS//' Qual#', IQAL, ':'
C
C Determine limits IPT:JPT of substring containing qualifier
        IF ( QALSTR(IPT:IPT) .NE. '(' ) THEN
          FAIL = .TRUE.
          ERRMSG = ERRSTR // 'read error. Expected ''('',  found ''' //
     &      QALSTR(IPT:IPT) // ''''
          RETURN
        END IF
        JPT = INDEX ( QALSTR(IPT+1:LPT), ')' )
        IF ( JPT .EQ. 0 ) THEN
          MPT = MIN ( LPT, IPT+9 )
          FAIL = .TRUE.
          ERRMSG =  ERRSTR // 'no '')'' found after ''' //
     &      QALSTR(IPT:MPT) // '...'''
          RETURN
        END IF
        JPT = JPT + IPT
C
C Read qualifier contents
        CALL REAQAL ( QALSTR(IPT:JPT), ONEQAL, ILS, IUS, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
C Single qualifier, interpreted as isotope#
        IF ( ONEQAL ) THEN 
          ISO = ILS
          IF ( ISO .EQ. 0 .OR. ABS ( ISO ) .GT. NISGAS ) THEN 
            FAIL = .TRUE.
            WRITE ( ERRMSG, '(A,I11,A,I4,A)' ) ERRSTR//
     &        'bad isotope#=', ISO, '  (range=1:', NISGAS, ')'
            RETURN
          END IF
          IF ( ISO .GT. 0 ) THEN 
            WRITE ( LOGMSG, '(A,I4)' ) 
     &        'I-GASQAL:   Select isotope#', ISO
          ELSE
            WRITE ( LOGMSG, '(A,I4)' ) 
     &        'I-GASQAL:   Deselect isotope#', ISO
          END IF
          CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
          IF ( LASBND ) THEN  
            LASBND = .FALSE.              ! modifying isotope list
            IF ( ISO .GT. 0 ) NISO = 0    ! start new list of isotopes
          END IF
          IF ( ISO .GT. 0 ) THEN  ! add isotope to list
            DO IISO = 1, NISO  ! check isotope not already in current list
              IF ( ISO .EQ. ISOLST(IISO) ) THEN
                FAIL = .TRUE.
                WRITE ( ERRMSG, '(A,I2)' ) ERRSTR//
     &            'repeated isotope#', ISO
                RETURN
              END IF
            END DO            
            NISO = NISO + 1
            ISOLST(NISO) = ISO
          ELSE                ! remove isotope from list
            JISO = 0
            DO IISO = 1, NISO
              IF ( ISOLST(IISO) .NE. ABS(ISO) ) THEN
                JISO = JISO + 1
                ISOLST(JISO) = ISOLST(IISO) 
              END IF
            END DO
            IF ( JISO .NE. NISO-1 ) THEN
              FAIL = .TRUE.
              WRITE ( ERRMSG, '(A,I2,A)' ) ERRSTR//
     &          'isotope#', ABS(ISO), ' not previously selected'
              RETURN
            END IF
            NISO = NISO - 1
          END IF  
C Two qualifiers, interpreted as low:high vib.level numbers
        ELSE
          IF ( ILS .LT. 0 .OR. ILS .GT. 999 ) THEN
            FAIL = .TRUE.
            WRITE ( ERRMSG, '(A,I11)' ) ERRSTR//
     &        'ILSGQ out of range 0:999, value=', ILS
            RETURN
          END IF
          IF ( IUS .LT. 0 .OR. IUS .GT. 999 ) THEN
            FAIL = .TRUE.
            WRITE ( ERRMSG, '(A,I11)' ) ERRSTR//
     &        'IUSGQ out of range 0:999, value=', IUS
            RETURN
          END IF
          LASBND = .TRUE.
C
          DO IISO = 1, NISO
            ISO = ISOLST(IISO)
C Check this isn't a repeated band,isotope combination
            DO IBND = 1, NBND(ISO,IGAS)
              FAIL = .TRUE.
              IF ( ILS .EQ. ILSQAL(IBND,ISO,IGAS) .AND.
     &             IUS .EQ. IUSQAL(IBND,ISO,IGAS) ) THEN
                WRITE ( ERRMSG, '(A,A,A,I2)' ) ERRSTR//
     &          'repeated band,isotope ', QALSTR(IPT:JPT), ',', ISO
              ELSE IF ( IUS .EQ. IUSQAL(IBND,ISO,IGAS) .AND. 
     &                  ILS * ILSQAL(IBND,ISO,IGAS) .EQ. 0 ) THEN
                WRITE ( ERRMSG, '(A,I2)' ) ERRSTR//
     &           'Inconsistent use of (*: ), Isotope#', ISO 
              ELSE IF ( ILS .EQ. ILSQAL(IBND,ISO,IGAS) .AND. 
     &                  IUS * IUSQAL(IBND,ISO,IGAS) .EQ. 0 ) THEN
                WRITE ( ERRMSG, '(A,I2)' ) ERRSTR//
     &           'Inconsistent use of ( :*), Isotope#', ISO 
              ELSE
                FAIL = .FALSE.
              END IF
              IF ( FAIL ) RETURN
            END DO
C
C Checks complete - load selected band and print info message to log file
            IF ( NBND(ISO,IGAS) .EQ. MAXBND ) THEN
              FAIL = .TRUE.
              WRITE ( ERRMSG, '(A,I11)' ) ERRSTR//
     &        'No.selected bands > MAXBND in rfmsiz.inc=', MAXBND
              RETURN
            END IF
            NBND(ISO,IGAS) = NBND(ISO,IGAS) + 1
            ILSQAL(IBND,ISO,IGAS) = ILS
            IUSQAL(IBND,ISO,IGAS) = IUS
          END DO
          LOGMSG = 'I-GASQAL:     Select lines from vib.band ' //
     &             'low:high=' // QALSTR(IPT:JPT)
          CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
        END IF
C
C Assume next character after ')' is start of next qualifier: '(' 
        IPT = JPT + 1
      END DO                           
C
C Successfully decoded entire QALSTR 
C
C Check if finished with an isotope list, in which case set ILS,IUS wildcards
      IF ( .NOT. LASBND ) THEN
        DO IISO = 1, NISO
          ISO = ISOLST(IISO)
          IF ( NBND(ISO,IGAS) .NE. 0 ) THEN
            FAIL = .TRUE.
            WRITE ( ERRMSG, '(A,I2)' ) ERRSTR//
     &      'bands already selected for isotope#', ISOLST(ISO)
            RETURN
          END IF
          NBND(ISO,IGAS) = 1
          ILSQAL(1,ISO,IGAS) = 0
          IUSQAL(1,ISO,IGAS) = 0
        END DO
        IF ( NISO .EQ. NISGAS ) THEN
          LOGMSG = 'W-GASQAL: all isotopes selected'
          CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        ELSE
          LOGMSG = 'I-GASQAL:     Select all bands'
          CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        END IF
        IF ( FAIL ) RETURN
C
      END IF
C
      FAIL = .FALSE.
C
      END 
