      SUBROUTINE GRDFIL ( LUNGRD, NAMGRD, FAIL, ERRMSG )
C
C VERSION
C     17NOV14 AD Major rewrite
C     17AUG06 AD Interpret -ve NPTS as grid info in GHz rather than cm-1
C     20MAR01 AD Add check on grid resolution for each spectral range
C     05DEC00 AD Comment out READONLY
C     05JUL99 AD Rename HEADER to CDUMMY, NUSE to IDUMMY.
C     18DEC97 AD Original.
C
C DESCRIPTION
C     Open GRD file and check contents
C     Called by INPGRD for each file specified in *GRD section of driver table.
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER       LUNGRD  !  I  LUN for GRD file
      CHARACTER*(*) NAMGRD  !  I  Name of GRD file
      LOGICAL       FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG  !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  NAMLOG ! Send text plus variable length filename to log file.
     &, REAGRD ! Read irregular grid data from .grd file
     &, RFMLOG ! Write text message to RFM log file.
C
C GLOBAL CONSTANTS
      INCLUDE 'phycon.inc' ! Physical and Mathematical constants
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C LOCAL CONSTANTS
      CHARACTER*39 FNCSTR 
        PARAMETER ( FNCSTR='lin qad cub 1li 1qa 1cu 1sq lor lnl lnc' )
      DOUBLE PRECISION WNOTOL ! Wavenumber tolerance as fraction of o/p resln
        PARAMETER ( WNOTOL = 0.01D0 )
      DOUBLE PRECISION WNOCNV ! Extra width [cm-1] to allow for any convolution
        PARAMETER ( WNOCNV = 1.0D0 )  ! Max ILS halfwidth=1.0
C
C COMMON VARIABLES
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'gflcom.inc' ! Grid file data
      INCLUDE 'grdcom.inc' ! Irregular grid 
      INCLUDE 'spccom.inc' ! Spectral Range & Resolution data
C
C LOCAL VARIABLES
      INTEGER      ISPC    ! Spectral range counter (1:NSPC)
      LOGICAL      CONVOL  ! T=applying convolution, F=no convolution
      LOGICAL      REJECT  ! T=ignore file - no useful data
      CHARACTER*80 MESSGE  ! Text message sent to LOG file
      DOUBLE PRECISION WNO1 ! Wavenumber [cm-1] of first grid pt (lowest Wno).
      DOUBLE PRECISION WNO2 ! Wavenumber [cm-1] of last grid pt (highest Wno).
C
C EXECUTABLE CODE -------------------------------------------------------------
C
      CONVOL = ILSFLG .OR. AVGFLG  
C
      CALL NAMLOG ( 'I-GRDFIL: Opening GRD file:', NAMGRD, FAIL, ERRMSG)
        IF ( FAIL ) RETURN
C
C 0.0d0 are spectral range, set to zero will just extract file header info
      CALL REAGRD ( LUNGRD, NAMGRD, 0.0D0, 0.0D0, FAIL, ERRMSG ) 
        IF ( FAIL ) RETURN
C
C Save spectral range of file since WNOGRD may be overwritten later
      WNO1 = WNOGRD(1)
      WNO2 = WNOGRD(2)
C
C Check if grid completely spans any selected output spectral range 
C NB: same grid may be usable for more than one spectral range
      REJECT = .TRUE.
      DO ISPC = 1, NSPC
        IF ( WNO1 .LE. WNLSPC(ISPC) .AND. 
     &       WNO2 .GE. WNUSPC(ISPC) ) THEN      ! matches this range
C
          FAIL = .TRUE.
          IF ( IDXGFL(ISPC) .NE. 0 ) THEN
            ERRMSG = 'F-GRDFIL: Another Grid already assigned '//
     &               'for spectral range:'//LABSPC(ISPC)
          ELSE IF ( NGFL .EQ. MAXGFL ) THEN
            WRITE ( ERRMSG, '(A,I11)' )
     &        'F-GRDFIL: Too many GRD files, increase array size '//
     &        'MAXGFL in rfmsiz.inc=', MAXGFL
          ELSE       ! re-read, this time including full grid info
            NGFL = NGFL + 1
            IDXGFL(ISPC) = NGFL
            NAMGFL(NGFL) = NAMGRD
            WMNGFL(NGFL) = WNO1
            WMXGFL(NGFL) = WNO2
            REJECT = .FALSE.
            MESSGE = 'I-GRDFIL: GRD assigned to spectral range:'//
     &               LABSPC(ISPC)
            CALL RFMLOG ( MESSGE, FAIL, ERRMSG )  ! use to set FAIL=FALSE
C If convolving, read in extra width to check MAXGRD array size not exceeded
            IF ( CONVOL ) THEN
              CALL REAGRD ( LUNGRD, NAMGRD, WNLSPC(ISPC)-WNOCNV, 
     &                      WNUSPC(ISPC)+WNOCNV, FAIL, ERRMSG )  
            ELSE
              CALL REAGRD ( LUNGRD, NAMGRD, WNLSPC(ISPC), 
     &                      WNUSPC(ISPC), FAIL, ERRMSG )  
            END IF
          END IF
          IF ( FAIL ) RETURN
C
C If applying convolution, all irreg grids must be at same resolution 
C (check this here), which is also same as the fine grid 
C (check in FINCHK in case it is modified by *FIN section).
          IF ( CONVOL ) THEN
            IF ( NGFL .EQ. 1 ) THEN
              RESGFL = WNRGRD
            ELSE
              IF ( ABS ( RESGFL - WNRGRD ) .GT. WNOTOL * WNRGRD ) THEN
                FAIL = .TRUE.
                ERRMSG = 'F-GRDFIL: With ILS or AVG convol,'//
     &                   'all irreg grids must be on same spacing' 
                RETURN
              END IF
            END IF
          ELSE
C if not using any spectral convolution, grid spacing must also match output
C spacing for this spectral range
            IF ( ABS ( WNRSPC(ISPC) - WNRGRD ) .GE. 
     &           WNOTOL * WNRGRD ) THEN    
              FAIL = .TRUE.
              ERRMSG = 'F-GRDFIL: grid does not match reqd resln'
              RETURN
            END IF
          END IF
        END IF
      END DO
C
      IF ( REJECT ) THEN
        MESSGE = 'W-GRDFIL: File ignored - grid not applicable'
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
      END IF
C
      END
