      SUBROUTINE INPSFC ( LUNDRV, LUNSFC, FAIL, ERRMSG ) 
C 
C VERSION 
C     10JUL15 AD Use C9FLT to construct error messages 
C     28NOV14 AD Rewritten, allow for field#1='REL' 
C     05SEP14 AD Remove LUNSFC argument from CHKFIL
C     13OCT13 AD Simplified: remove GETSFC argument and local GOTSFC checks
C     07AUG13 AD Change FORMAT descriptors to avoid ifort warnings
C     23APR12 AD Allow for spectrally varying emissivity
C                Add LUNSFC argument. 
C     23JAN02 AD Increase precision in info message G10.3 to G10.5
C     14JUL98 AD Original.  
C 
C DESCRIPTION 
C     Read RFM surface parameters from driver table 
C     Called by RFMINP following *SFC marker in Driver table.  
C     If SFCFLG is TRUE read surface temperature and emissivity.  
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER      LUNDRV  !  I  LUN for driver table
      INTEGER      LUNSFC  !  I  Temporary LUN for surface emissivity file
      LOGICAL      FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80 ERRMSG  !  O  Error message written if FAIL is TRUE
C
      EXTERNAL
     &  C9FLT  ! Write real number as C*9 string up to 3 s.f.
     &, CHKFIL ! Check if argument represents an existing filename
     &, ENDCHK ! Check end of Driver Table section has been reached.
     &, NXTFLD ! Load next field from section of RFM driver file
     &, RFMLOG ! Write text message to RFM log file
     &, SFCFIL ! Open, read surface emissivity data file and close
     &, UPCASE ! Convert text string to upper case.
      CHARACTER*9 C9FLT
      LOGICAL CHKFIL
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C COMMON VARIABLES
      INCLUDE 'atmcom.inc' ! Atmospheric profile data
      INCLUDE 'sfccom.inc' ! Surface parameters
C
C LOCAL CONSTANTS
      REAL          TEMDEL ! Max. |SFC-ATM| Temp.difference before warning
        PARAMETER ( TEMDEL = 10.0 )       ! arbitrary figure
      REAL          TEMMIN ! Minimum allowed surface temperature
        PARAMETER ( TEMMIN = 0.0 )
      REAL          TEMMAX ! Maximum allowed surface temperature
        PARAMETER ( TEMMAX = 1000.0 )     ! arbitrary figure
C
C LOCAL VARIABLES
      INTEGER       IOS    !  Saved value of IOSTAT for error message
      INTEGER       LENGTH !  No.of characters written in FIELD
      LOGICAL       LRELTV !  T=Temp relative to lower atm temp, F=absolute
      CHARACTER*200 FIELD  !  Field read from driver table
      CHARACTER*80  MESSGE !  Message sent to log file
      CHARACTER*80  UFIELD ! Upper case version of FIELD
C
C EXECUTABLE CODE --------------------------------------------------------------
C
C Read first field from *SFC section
      CALL NXTFLD ( LUNDRV, FIELD, LENGTH, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
C
C First field could be 'REL', in which case load next field for temperature
      CALL UPCASE ( FIELD, UFIELD )  ! also works if LENGTH=0
      LRELTV = UFIELD .EQ. 'REL'
      IF ( LRELTV ) THEN
        CALL NXTFLD ( LUNDRV, FIELD, LENGTH, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      END IF
C
C Field should now contain a temperature value, either absolute or relative
      IF ( LENGTH .EQ. 0 ) THEN
        ERRMSG = 'F-INPSFC: No Surface Temperature field '//
     &           'in *SFC section'
        FAIL = .TRUE.
        RETURN
      END IF
C
C Read numerical value
      READ ( FIELD, *, IOSTAT=IOS ) TEMSFC
      IF ( IOS .NE. 0 ) THEN
        WRITE ( ERRMSG, '(A,I11)' )
     &    'F-INPSFC: Error reading Surface Temperature.'
     &    //' IOSTAT=', IOS
        FAIL = .TRUE.
        RETURN
      END IF
C
C Note value read in RFM runlog file
      MESSGE = 'I-INPSFC: Reading Surface Temperature = '//
     &  C9FLT(TEMSFC)//' K'
      CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
C
C If relative, then also note adjusted temperature in runlog file
      IF ( LRELTV ) THEN
        TEMSFC = TEMATM(1) + TEMSFC
        MESSGE = 'I-INPSFC: Add to lower atm.temp='//C9FLT(TEMATM(1))
     &    //' K gives Tsfc='//C9FLT(TEMSFC)//' K' 
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      END IF
C
C Check if within allowed range
      IF ( TEMSFC .LT. TEMMIN .OR. TEMSFC .GT. TEMMAX ) THEN       
        FAIL = .TRUE.
        WRITE ( ERRMSG, '(A,F8.0,A,F8.0)' )
     &     'F-INPSFC: Surface Temperature outside allowed range: ',
     &     TEMMIN, ' to ', TEMMAX
        RETURN
      END IF
C
C Warn if large surface-atmosphere temperature discontinuity
      IF ( ABS ( TEMSFC-TEMATM(1) ) .GT. TEMDEL ) THEN
        MESSGE = 'W-INPSFC: Large SFC-ATM Temperature discontinuity, ='
     &   //C9FLT(TEMSFC-TEMATM(1))//' K'
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      END IF
C
C Set default surface emissivity to be black body
      NSFC = 1
      WNOSFC(1) = 0.0D0
      EMSSFC(1) = 1.0D0
C
C Next field (optional) is Surface Emissivity or file of emissivity values
      CALL NXTFLD ( LUNDRV, FIELD, LENGTH, FAIL, ERRMSG )
      IF ( LENGTH .EQ. 0 ) THEN             ! assume default BB surface
        MESSGE = 'I-INPSFC: Assuming default Surface Emissivity=1'
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      ELSE IF ( CHKFIL ( FIELD ) ) THEN 
        CALL SFCFIL ( LUNSFC, FIELD, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
      ELSE
        READ ( FIELD, *, IOSTAT=IOS ) EMSSFC(1)
        IF ( IOS .NE. 0 ) THEN
          FAIL = .TRUE.
          WRITE ( ERRMSG, '(A,I11)' )
     &      'F-INPSFC: Error reading Surface Emissivity value.'
     &    //' IOSTAT=', IOS
          RETURN
        END IF
        MESSGE = 'I-INPSFC: Setting Surface Emissivity = '//
     &     C9FLT ( SNGL ( EMSSFC(1) ) )
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
        IF ( EMSSFC(1) .LT. 0.D0 .OR. EMSSFC(1) .GT. 1.D0 ) THEN       
          FAIL = .TRUE.
          ERRMSG = 
     &      'F-INPSFC: Surface Emissivity outside range 0:1'
          RETURN
        END IF
      END IF
C
C Set flag for reflecting surface
      RFLSFC = ( NSFC .GT. 1 .OR. EMSSFC(1) .LT. 1.0D0 )
C
C Check that there are no more fields in this section
      IF ( LENGTH .GT. 0 ) 
     &  CALL ENDCHK ( LUNDRV, 'SFC', FAIL, ERRMSG )
C
      END
