      SUBROUTINE INPSHP ( LUNDRV, FAIL, ERRMSG )
C
C VERSION
C     05DEC14 AD Rewritten. 
C                Assume all lines have default Voigt shape already set
C                Allow for molecules to be specified by index.
C                Use CHKGAS to check for valid molecule name
C     17OCT13 AD Simplified: remove GETSHP argument and local GOTSHP checks
C     11AUG03 AD Add VVW shape
C     16FEB02 AD Allow for different isotopes 
C     31DEC98 AD Comment change only
C     01JUN98 AD Correct warning message for CO2: warn if CTM *not* 
C                  combined with chi-factor 
C     04DEC97 AD Add GOTSHP to check for duplicate section
C     01DEC97 AD Check for SHPCTM, also legitimate continuum shapes.
C                Send info message to LOG file about shapes being used.
C     01SEP96 AD Version 1.
C
C DESCRIPTION
C     Read RFM line shapes from driver table *SHP section.
C     Called by RFMINP once if SHPFLG set TRUE.
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER      LUNDRV  !  I  LUN for Driver File
      LOGICAL      FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80 ERRMSG  !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  CHKGAS ! Check if GAS is valid molecule and get Hitran/RFM index
     &, LOCASE ! Convert text string to lower case.
     &, NXTREC ! Load next record from RFM input file.
     &, RFMLOG ! Write text message to RFM log file.
     &, TXTFLD ! Identify start and end points of text field in record
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
      INCLUDE 'idxcon.inc' ! HITRAN molecular indices
      INCLUDE 'shpcon.inc' ! Line Shape codes
C
C COMMON VARIABLES
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'gascom.inc' ! Molecular and isotope data
C
C LOCAL VARIABLES
      INTEGER      IEND    ! Pointer to end of filename in record
      INTEGER      IFLD    ! Text Field# within record
      INTEGER      IGAS    ! Absorber counter
      INTEGER      IGQ     ! Global quantum index (should be zero)
      INTEGER      IMOL    ! HITRAN/RFM Molecule index
      INTEGER      ISHP    ! Line shape index#
      INTEGER      ISO     ! Isotope counter
      INTEGER      ISTA    ! Pointer to start of filename in record
      INTEGER      JGAS    ! Index of isotopes in *GAS arrays
      INTEGER      SHPDEF  ! Default Line shape to be used
      LOGICAL      ENDSEC  ! T=End of driver table section reached.
      LOGICAL      GOTGAS(MAXGAS) ! T=shape specified for molecule
      CHARACTER*7  FIELD   ! Absorber name (C*7 is longest name in use)
      CHARACTER*7  GAS     ! Molecule name
      CHARACTER*80 LOGMSG  ! Text message for Log file
      CHARACTER*80 RECORD  ! Line of Driver file or External TAN file
      CHARACTER*3  SHAPE   ! Line shape name
C
C DATA STATEMENTS
      DATA GOTGAS / MAXGAS * .FALSE. /
C
C EXECUTABLE CODE -------------------------------------------------------------
C
C Each record should start with a recognised lineshape (VOIGT,LORENTZ etc)
      SHPDEF = 0
      CALL NXTREC ( LUNDRV, RECORD, ENDSEC, FAIL, ERRMSG ) 
      IF ( FAIL ) RETURN
      DO WHILE ( .NOT. ENDSEC ) 
        CALL TXTFLD ( 80, RECORD, 1, ISTA, IEND )    ! Locate Field#1 (shape) 
        IEND = MIN ( ISTA+2, IEND )                  ! Limit to C*3
        SHAPE = RECORD(ISTA:IEND)
        CALL LOCASE ( SHAPE, SHAPE )      
        IF ( SHAPE .EQ. 'voi' ) THEN
          ISHP = SHPVOI
        ELSE IF ( SHAPE .EQ. 'lor' ) THEN
          ISHP = SHPLOR
        ELSE IF ( SHAPE .EQ. 'dop' ) THEN
          ISHP = SHPDOP
        ELSE IF ( SHAPE .EQ. 'chi' ) THEN
          ISHP = SHPCHI
        ELSE IF ( SHAPE .EQ. 'vvw' .OR. SHAPE .EQ. 'van') THEN
          ISHP = SHPVVW
        ELSE
          FAIL = .TRUE.                                  
          ERRMSG = 'F-INPSHP: Unrecognised Line Shape: '//SHAPE
          RETURN
        END IF
C
        IF ( ISHP .EQ. SHPVVW .AND. VVWFLG ) THEN 
          LOGMSG = 'W-INPSHP: VVW correction flg will be ignored '//
     &             'for lines calculated with VVW shape'
          CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
        END IF
C
C Subsequent fields in same record should contain list of gases with spec.shape
        IFLD = 2
        CALL TXTFLD ( 80, RECORD, IFLD, ISTA, IEND )
        DO WHILE ( ISTA .GT. 0 ) 
          FIELD = RECORD(ISTA:IEND)
          IF ( FIELD .EQ. '*' ) THEN
            IF ( SHPDEF .EQ. 0 ) THEN
              SHPDEF = ISHP
              LOGMSG = 'I-INPSHP: Setting default '//SHAPE//
     &          ' lineshape for any remaining gases'
              CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
              IF ( FAIL ) RETURN	   
            ELSE
              ERRMSG = 'F-INPSHP: Two different default line shapes '//
     &                 'specified'
              FAIL = .TRUE.
              RETURN
            END IF
          ELSE        ! assume molecule name
            CALL CHKGAS ( FIELD, GAS, IMOL, ISO, IGQ, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
            IF ( IMOL .EQ. 0 ) THEN
              ERRMSG = 'F-INPSHP: Unrecognised molecule='//FIELD
              FAIL = .TRUE.
              RETURN
            END IF
            IGAS = IGSMOL(IMOL) 
            FAIL = .TRUE.           
            IF ( ISO .NE. 0 .OR. IGQ .NE. 0 ) THEN
              ERRMSG = 'F-INPSHP: Qualifiers not allowed after '//
     &                 'molecule names in *SHP section' 
            ELSE IF ( IGSMOL(IMOL) .EQ. 0 ) THEN
              ERRMSG = 'F-INPSHP: Line Shape specified for '//
     &                 'unlisted gas= '//GAS
            ELSE IF ( GOTGAS(IGAS) ) THEN
              ERRMSG = 'F-INPSHP: Two different line shapes '//
     &                 'specified for gas= '//GAS
            ELSE IF ( SHPGAS(IGAS) .EQ. SHPCTM ) THEN
              ERRMSG = 'F-INPSHP: line shape specified for '//
     &            GAS//' but already set to continuum-only'
            ELSE IF ( SHPGAS(IGAS) .EQ. SHPXSC ) THEN
              ERRMSG = 'F-INPSHP: line shape specified for '//
     &            GAS//' but already set as cross-section'
            ELSE
              FAIL = .FALSE.
            END IF
            IF ( FAIL ) RETURN
            SHPGAS(IGAS) = ISHP
            IF ( CHIFLG .AND. IMOL .EQ. IDXCO2 ) THEN
              LOGMSG = 'W-INPSHP: Overwriting CO2 Chi-factor '//
     &                 'set by CHI flag'
              CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
            END IF
            GOTGAS(IGAS) = .TRUE.
            DO ISO = 1, NISGAS(IGAS)        ! Also set any other isotopes
              JGAS = ISOGAS(ISO,IGAS)
              SHPGAS(JGAS) = SHPGAS(IGAS)
              GOTGAS(JGAS) = .TRUE.
            END DO
            LOGMSG = 'I-INPSHP: Using '//SHAPE
     &            //' lineshape for gas='//GAS
            CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
          END IF                         ! decode molec or '*' field in record
          IFLD = IFLD + 1
          CALL TXTFLD ( 80, RECORD, IFLD, ISTA, IEND )
        END DO
        CALL NXTREC ( LUNDRV, RECORD, ENDSEC, FAIL, ERRMSG ) 
        IF ( FAIL ) RETURN
      END DO
C
C Reached end of section - assign default shape to any gases not yet assigned
C Assume all line molecules have Voigt shape already set by default so use 
C this to identify molecules where other lineshapes are applicable
C If CHI flag used, CO2 will have been assigned CHI lineshape so leave unchanged
      IF ( SHPDEF .GT. 0 ) THEN
        LOGMSG = 'I-INPSHP: Using default lineshape for gases:'
        CALL ACCLOG ( LOGMSG, FAIL, ERRMSG )
        DO IGAS = 1, NGAS
          IF ( .NOT. GOTGAS(IGAS) .AND. SHPGAS(IGAS) .EQ. SHPVOI ) THEN
            SHPGAS(IGAS) = SHPDEF               ! ..so use default shape
            CALL ACCLOG ( CODGAS(IGAS), FAIL, ERRMSG )
            IF ( FAIL ) RETURN
          END IF
        END DO
        CALL ACCLOG ( ' ', FAIL, ERRMSG )
      END IF
C
C Issue warning if inconsistent shape + continuum used for CO2 or H2O
      DO IGAS = 1, NGAS
        IF ( CTMGAS(IGAS) .AND. SHPGAS(IGAS) .NE. SHPCTM ) THEN
          IF ( IDXGAS(IGAS) .EQ. IDXCO2 ) THEN
            IF ( SHPGAS(IGAS) .NE. SHPCHI ) THEN
              LOGMSG = 'W-INPSHP: CHI-FACTOR recommended for'//
     &                 ' use with CO2 continuum'  
              CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
              IF ( FAIL ) RETURN
            END IF
          ELSE IF ( IDXGAS(IGAS) .EQ. IDXH2O ) THEN
            IF ( SHPGAS(IGAS) .NE. SHPLOR .AND. 
     &           SHPGAS(IGAS) .NE. SHPVOI       ) THEN
              LOGMSG = 'W-INPSHP: LORENTZ or VOIGT line-shape'//
     &                 ' recommended for use with H2O continuum'
              CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
              IF ( FAIL ) RETURN
            END IF
          END IF
        END IF
C Currently, Chi-factor only implemented for CO2
        IF ( SHPGAS(IGAS) .EQ. SHPCHI .AND. 
     &       IDXGAS(IGAS) .NE. IDXCO2 ) THEN
          FAIL = .TRUE. 
          ERRMSG = 'F-INPSHP: Chi-factor only appicable to CO2'
          RETURN
        END IF
      END DO
C
      FAIL = .FALSE.
      END
