      SUBROUTINE LUTFIL ( LUNLUT, NAMLUT, FAIL, ERRMSG )
C
C VERSION
C     09APR15 AD Bug#100: RETURN if MAXLUX is exceeded
C     13FEB15 AD Use REAQAL instead of LUTQAL
C     26NOV14 AD Set IRRLFL
C     05SEP14 AD Use INQUIRE to test if file exists before finding type.
C     16JAN14 AD Rewritten 
C     04APR06 AD Change 'X' to '1X' in format 
C     25OCT04 AD Bug Fix: Check isotopic LUTs actually required
C     26MAR04 AD Allow for isotopic LUTs
C     21JUN01 AD Assume binary if ".bin" or ".BIN" in filename 
C     05DEC00 AD Comment out READONLY
C     03JUL97 AD Original.
C
C DESCRIPTION
C     Open Look-Up Table file and check contents
C     Called by INPLUT for each file specified in *LUT section of driver table.
C     Keeps LUN open for all usable LUT files.
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER       LUNLUT  ! I/O LUN for LUT File/next free LUN
      CHARACTER*(*) NAMLUT  ! I/O Name of LUT file (+optional qualifier)
      LOGICAL       FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG  !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  LUTSVD ! Load SVD-compressed LUT data from file.
     &, LUTSPC ! Determine spectral range(s) to which LUT can be applied
     &, LUTTAB ! Open TAB-LUT file and read in header info.
     &, NAMLOG ! Send text plus variable length filename to log file.
     &, REAQAL ! Decode Qualifier strings '(I1)' or '(I1:I2)'
     &, RFMLOG ! Write text message to RFM log file.
     &, TYPLUT ! Identify type of LUT file.
      CHARACTER*3 TYPLUT
C
C GLOBAL CONSTANTS
      INCLUDE 'phycon.inc' ! Physical and Mathematical constants
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C COMMON VARIABLES
      INCLUDE 'lflcom.inc' ! Look-Up Table file data
      INCLUDE 'svdcom.inc' ! SVD-compressed Look-Up Table data
      INCLUDE 'tabcom.inc' ! Table axes for tabulated absorb.coeffs.
C
C LOCAL CONSTANTS
      DOUBLE PRECISION WNOTOL    ! Waveno. tolerance as fraction of Spc grid
        PARAMETER ( WNOTOL = 0.01D0 ) 
C
C LOCAL VARIABLES
      INTEGER       IGAS   ! Index for /GASCOM/ 
      INTEGER       IQAL   ! First value in QAL string
      INTEGER       JQAL   ! Second value in QAL string (if more than 1)
      INTEGER       IPT    ! Pointer to '(' marking start of qualifier data
      INTEGER       LPT    ! Length of original NAMLUT string
      INTEGER       LUNSVD ! LUN assigned for SVD LUTs, or 0 if not assigned
      INTEGER       NDP    ! Sub-Sampling for p-axis
      INTEGER       NDT    ! Sub-Sampling for T-axis
      INTEGER       NL     ! No.of basis vectors file
      LOGICAL       ANYQAL ! T=qualifiers appended to end of file name
      LOGICAL       BINFIL ! T=binary file, F=ASCII file
      LOGICAL       LEXIST ! T=file exists, F=file doesn't exist
      LOGICAL       ONEQAL ! T=only one value in QAL string, F=two values
      LOGICAL       REJECT ! T=ignore file - no useful data
      CHARACTER*14  QAL    ! Qualifier string (if any) appended to filename
      CHARACTER*3   TYP    ! Type of LUT file: 'ASC', 'BIN' or 'SVD'
      CHARACTER*80  WRNMSG ! Text of warning message sent to log file
      DOUBLE PRECISION V1  ! Lower wavenumber limit of LUT [/cm]
      DOUBLE PRECISION V2  ! Upper wavenumber limit of LUT [/cm]
C
C DATA STATEMENTS
      DATA LUNSVD / 0 /
      SAVE LUNSVD
C
C EXECUTABLE CODE -------------------------------------------------------------
C
      CALL NAMLOG ( 'I-LUTFIL: Checking LUT file:', NAMLUT, FAIL,ERRMSG)
      IF ( FAIL ) RETURN
C
C Separate any qualifier string from the filename
      ANYQAL = INDEX ( NAMLUT, '(' ) .GT. 1
      IF ( ANYQAL ) THEN
        IPT = INDEX ( NAMLUT, '(' )
        LPT = LEN ( NAMLUT )
        QAL = NAMLUT(IPT:LPT)
        NAMLUT = NAMLUT(1:IPT-1)
      END IF
C
      INQUIRE ( FILE=NAMLUT, EXIST=LEXIST )
      IF ( .NOT. LEXIST ) THEN
        FAIL = .TRUE.
        ERRMSG = 'F-LUTFIL: file does not exist'
        RETURN
      END IF
      TYP = TYPLUT ( LUNLUT, NAMLUT )
      IF ( TYP .EQ. ' ' ) THEN
        FAIL = .TRUE.
        ERRMSG = 'F-LUTFIL: Unable to identify type of LUT file'
        RETURN
      END IF
C
      NDP = 1
      NDT = 1
      NL = MAXLUL + 1
      IF ( ANYQAL ) THEN
        CALL REAQAL ( QAL, ONEQAL, IQAL, JQAL, FAIL, ERRMSG ) 
        IF ( FAIL ) RETURN
        IF ( TYP .EQ. 'SVD' ) THEN
          IF ( .NOT. ONEQAL ) THEN
            ERRMSG = 'F-LUTQAL: Only one qualifier expected for SVD LUT'
            FAIL = .TRUE.
            RETURN
          END IF
          NL = IQAL
        ELSE                     ! TYP = 'ASC' or 'BIN', so .tab file
          IF ( ONEQAL ) THEN
            ERRMSG = 'F-LUTQAL: Two qualifiers expected for TAB LUT'
            FAIL = .TRUE.
            RETURN
          END IF 
          NDP = IQAL
          NDT = JQAL
        END IF
      END IF
C
      IGAS = 0
      IF ( TYP .EQ. 'SVD' ) THEN 
        CALL LUTSVD ( LUNLUT, NAMLUT, 1, NL, IGAS, FAIL, ERRMSG )
      ELSE
        NL = 0
        BINFIL = TYP .EQ. 'BIN'
C Don't subsample here since require NPTAB, NTTAB at original size for NX calc
        CALL LUTTAB ( LUNLUT, NAMLUT, BINFIL, IGAS, 1, 1, FAIL, ERRMSG )
      END IF
      IF ( FAIL ) RETURN
C
      IF ( IGAS .EQ. 0 ) THEN 
        WRNMSG = 'W-LUTFIL: LUT ignored - molec/iso not required'
        CALL RFMLOG ( WRNMSG, FAIL, ERRMSG )
        RETURN
      ENDIF
C
C Extract wavenumber range of LUT file from loaded header info
      IF ( TYP .EQ. 'SVD' ) THEN
        V1 = V1SVD(1)
        V2 = V2SVD(1)
      ELSE
        V1 = V1TAB
        V2 = V2TAB
        IF ( NVTAB .LT. 0 ) THEN   ! v1tab, v2tab in GHz
          V1 = V1 * 1.0D7 / VLIGHT
          V2 = V2 * 1.0D7 / VLIGHT
        END IF
      END IF
C
C Check if this spectral range is required
      CALL LUTSPC ( V1, V2, IGAS, REJECT, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
      IF ( REJECT ) THEN
        WRNMSG = 'W-LUTFIL: LUT ignored - does not cover any'//
     &           'required spectral ranges'
        CALL RFMLOG ( WRNMSG, FAIL, ERRMSG )
        RETURN 
      END IF
C
      IF ( NL .GT. MAXLUL ) THEN
        FAIL = .TRUE.
        WRITE ( ERRMSG, '(A,I11,A,I6)' ) 
     &    'F-LUTFIL: No.S.Vs =', NL, ' > MAXLUL in rfmsiz.inc=', MAXLUL
        RETURN
      END IF
C
      IF ( NLFL .EQ. MAXLFL ) THEN
        FAIL = .TRUE.
        ERRMSG = 'F-LUTFIL: Total number of LUT files '//
     &           '> MAXLFL in rfmsiz.inc'
        RETURN
      END IF
      NLFL = NLFL + 1
C
      IGSLFL(NLFL) = IGAS
      NDPLFL(NLFL) = NDP
      NDTLFL(NLFL) = NDT
      NLLFL(NLFL) = NL
      SVDLFL(NLFL) = TYP .EQ. 'SVD'
      BINLFL(NLFL) = TYP .EQ. 'BIN'
      NAMLFL(NLFL) = NAMLUT
      V1LFL(NLFL) = V1
      V2LFL(NLFL) = V2
C
      IF ( TYP .EQ. 'SVD' ) THEN
        NVLFL(NLFL) = NVSVD(1)
        NXLFL(NLFL) = NPSVD(1) * NTSVD(1)
      ELSE
        GHZLFL(NLFL) = NVTAB .LT. 0
        NVLFL(NLFL) = ABS ( NVTAB )
        NXLFL(NLFL) = NPTAB * NTTAB * NQTAB    ! value before axis subsampling
        NPLFL(NLFL) = NPTAB
        NQLFL(NLFL) = NQTAB
        NTLFL(NLFL) = NTTAB
        IRRLFL(NLFL) = 1 + NINT ( ( V2TAB - V1TAB ) / DVTAB ) .NE. NVTAB
      END IF
      IF ( NXLFL(NLFL) .GT. MAXLUX ) THEN
        FAIL = .TRUE.
        WRITE ( ERRMSG, '(A,I11,A,I6)' )
     &    'F-LUTFIL: No p*T points in file=', NXLFL(NLFL), 
     &    ' > MAXLUX in rfmsiz.inc=', MAXLUX
        RETURN
      END IF
C
C Can re-use same LUN for all SVD LUTs
      IF ( TYP .EQ. 'SVD' .AND. LUNSVD .GT. 0 ) THEN
        LUNLFL(NLFL) = LUNSVD
      ELSE
        IF ( TYP .EQ. 'SVD' ) LUNSVD = LUNLUT
        LUNLFL(NLFL) = LUNLUT
        LUNLUT = LUNLUT + 1
      END IF
C
      FAIL = .FALSE.
      RETURN
C
      END
