      SUBROUTINE LUTGRD ( ISPC, FAIL, ERRMSG )
C
C VERSION
C     13JUL15 AD Add NAMGAS
C     09DEC14 AD Original
C
C DESCRIPTION
C     Use LUT files to set irregular grid.
C     Called once by INPCHK if GRD and LUT flags enabled.
C     and for each spectral range by SPCGRD where grid derived from LUTs.
C     The grid can only be derived if LUTs are specified for every absorber.
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER       ISPC   !  I  0=all ranges, >0=specific spectral range
      LOGICAL       FAIL   !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  LUTTAB ! Open TAB-LUT file and read in header info.
     &, NAMGAS ! Return molecular name associated with /GASCOM/ index
     &, RFMLOG ! Write text message to RFM log file.
      CHARACTER*7 NAMGAS
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C COMMON VARIABLES
      INCLUDE 'fincom.inc' ! Fine mesh data
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'gascom.inc' ! Molecular and isotope data
      INCLUDE 'gflcom.inc' ! Grid file data
      INCLUDE 'grdcom.inc' ! Irregular grid 
      INCLUDE 'lflcom.inc' ! Look-Up Table file data
      INCLUDE 'spccom.inc' ! Spectral Range & Resolution data
C
C LOCAL VARIABLES
      INTEGER      IGAS    ! Index of absorber in /GASCOM/
      INTEGER      IL      ! Length of GASSTR
      INTEGER      ILFL    ! Index of LUT file in /LFLCOM/
      INTEGER      IOS     ! Saved value of IOSTAT for error messages
      INTEGER      ISPC1,ISPC2 ! Start/End values of spc.range loop
      INTEGER      IX      ! Counter for LUT k-value points
      INTEGER      JLFL    ! Index of LUT data in /LFLCOM/ for next read
      INTEGER      JSPC    ! Counter for spectral ranges in /SPCCOM/
      INTEGER      NFUL    ! Number of full grid points reqd for spectral range
      REAL         RDUMMY  ! Dummy variable for reading past k-value points
      CHARACTER*7  GASSTR  ! Molecule name (+any isotopic ID)
      CHARACTER*80 MESSGE  ! Message sent to log file
C
      DOUBLE PRECISION WNOLFL(MAXLFL) ! Current Wno [cm-1] from each LUT file
      DOUBLE PRECISION WNOLUT ! Wavenumber [cm-1] of record from LUT file
      DOUBLE PRECISION WNOMAX ! Highest wno [cm-1] required for spectral range
      DOUBLE PRECISION WNOMIN ! Lowest  wno [cm-1] required for spectral range
      DOUBLE PRECISION WNOTOL ! Tolerance [cm-1] for matching spectral values
C
C EXECUTABLE CODE --------------------------------------------------------------
C
      IF ( ISPC .EQ. 0 ) THEN
        ISPC1 = 1
        ISPC2 = NSPC
      ELSE
        ISPC1 = ISPC
        ISPC2 = ISPC
      END IF
C
      DO JSPC = ISPC1, ISPC2
C Check if grid for this spectral range already obtained from .grd file
        IF ( IDXGFL(JSPC) .GT. 0 ) GOTO 100   ! Irreg.grid file already assigned
C
C Try to build irregular grid from LUT files
        WRITE ( MESSGE, '(A,A,A)' ) 
     &   'I-LUTGRD: Spectral range: ', LABSPC(JSPC), 
     &   ' building irreg. spectral grid from LUTs'
        CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
          IF ( FAIL ) RETURN 
C
        DO IGAS = 1, NGAS
          GASSTR = NAMGAS ( IGAS, IL )
          ILFL = IDXLFL(IGAS,JSPC) 
C Check if all absorber cross-sections derived from LUT files
          IF ( ILFL .EQ. 0 ) THEN
            MESSGE = 'W-LUTGRD: No LUT for ' // GASSTR(1:IL)
     &        // ', so full grid used'
            CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
              IF ( FAIL ) RETURN
            GOTO 100
C Check if any LUT files are specified on a regular grid
          ELSE IF ( .NOT. IRRLFL(ILFL) ) THEN 
            MESSGE = 'W-LUTGRD: LUT for ' // GASSTR(1:IL) //
     &        ' regularly spaced, so full grid used'
            CALL RFMLOG ( MESSGE, FAIL, ERRMSG )
              IF ( FAIL ) RETURN
            GOTO 100
          END IF
        END DO
C
        WNOMIN = WXLSPC(JSPC)
        WNOMAX = WXUSPC(JSPC)
        FNCGRD = 'lin'
        TWOGRD = INDEX ( 'lin 1li 1sq lor lnl', FNCGRD ) .NE. 0
C
C Start by advancing through each LUT file to first point past start of 
C spectral range and note file with the lowest wavenumber 
C Establish wavenumber tolerance for matching grid points
        IF ( ILSFLG .OR. AVGFLG ) THEN 
          WNOTOL = (1.0D0/NOMFIN) * 0.1
        ELSE
          WNOTOL = WNRSPC(JSPC) * 0.1
        END IF
        WNOLUT = 2 * WNOMAX
        DO IGAS = 1, NGAS
          ILFL = IDXLFL(IGAS,JSPC) 
C Open .tab file and skip file header
          CALL LUTTAB ( LUNLFL(ILFL), NAMLFL(ILFL), BINLFL(ILFL),
     &                  IGAS, 1, 1, FAIL, ERRMSG ) 
            IF ( FAIL ) RETURN
          WNOLFL(ILFL) = 0.0D0
          DO WHILE ( WNOLFL(ILFL) .LT. WNOMIN + WNOTOL )
            IF ( BINLFL(ILFL) ) THEN
              READ ( LUNLFL(ILFL), IOSTAT=IOS, ERR=900 ) 
     &          WNOLFL(ILFL), ( RDUMMY, IX = 1, NXLFL(ILFL) )
            ELSE
              READ ( LUNLFL(ILFL), *, IOSTAT=IOS, ERR=900 ) 
     &          WNOLFL(ILFL), ( RDUMMY, IX = 1, NXLFL(ILFL) )
            END IF
          END DO
          IF ( WNOLFL(ILFL) .LT. WNOLUT ) THEN
            JLFL = ILFL
            WNOLUT = WNOLFL(ILFL)
          END IF
        END DO
C
C Set first point in grid at start of spectral range
        WNOGRD(1) = WNOMIN
        NGRD = 1 
C
C Main loop: find next wno.from LUT files until all beyond end of spc.range
        DO WHILE ( WNOLUT .LT. WNOMAX + WNOTOL )
C
C Add grid point if within spectral range and above previously set grid point
          IF ( WNOLUT .GT. WNOGRD(NGRD)+WNOTOL ) THEN
            IF ( NGRD .EQ. MAXGRD ) THEN
              FAIL = .TRUE. 
              WRITE ( ERRMSG, '(A,I11)' ) 'F-LUTGRD: No.irreg '//
     &              'grid pts > MAXGRD in rfmsiz.inc=', MAXGRD
              RETURN
            END IF
            NGRD = NGRD + 1
            WNOGRD(NGRD) = WNOLUT
          END IF
C
C Make sure we don't read beyond the end of a file
          IF ( ABS ( WNOLFL(JLFL) - V2LFL(JLFL) ) .LT. WNOTOL ) THEN  ! eof
            WNOLFL(JLFL) = 2 * WNOMAX
          ELSE IF ( BINLFL(JLFL) ) THEN 
            READ ( LUNLFL(JLFL), IOSTAT=IOS, ERR=900 ) 
     &        WNOLFL(JLFL), ( RDUMMY, IX = 1, NXLFL(JLFL) )
          ELSE
            READ ( LUNLFL(JLFL), *, IOSTAT=IOS, ERR=900 ) 
     &        WNOLFL(JLFL), ( RDUMMY, IX = 1, NXLFL(JLFL) )
          END IF
C
C Find which LUT has the lowest next wavenumber
          WNOLUT = 2 * WNOMAX
          DO IGAS = 1, NGAS
            ILFL = IDXLFL(IGAS,JSPC) 
            IF ( WNOLFL(ILFL) .LT. WNOLUT ) THEN 
              JLFL = ILFL
              WNOLUT = WNOLFL(ILFL)
            END IF
          END DO
C
        END DO
C
C Close all LUTs
        DO IGAS = 1, NGAS
          ILFL = IDXLFL(IGAS,JSPC) 
          CLOSE ( LUNLFL(ILFL), IOSTAT=IOS, ERR=900 ) 
        END DO
C
        IF ( AVGFLG .OR. ILSFLG ) THEN
          NFUL = 1 + NINT ( ( WNOMAX - WNOMIN ) * NOMFIN ) 
          WRITE ( MESSGE, '(A,F7.2,A)' ) 'I-LUTGRD: Irreg.grid uses',
     &      100.0 * FLOAT(NGRD)/FLOAT(NFUL), 
     &      '% of full spectral grid'
          CALL RFMLOG ( MESSGE, FAIL, ERRMSG ) 
            IF ( FAIL ) RETURN
        END IF
        IDXGFL(JSPC) = -1  ! Flag spc range grid to be derived from LUTs
 100    CONTINUE
      END DO
C Normal exit
      FAIL = .FALSE.
      RETURN

C Exit with Fatal I/O error
 900  CONTINUE
      WRITE ( ERRMSG, '(A,I11)' ) 
     &  'F-LUTGRD: I/O failure reading LUT file, IOSTAT=', IOS
      FAIL = .TRUE.
C
      END
