      SUBROUTINE LUTTAB ( LUNLUT, NAMLUT, BINFIL, JGAS, NDP, NDT, 
     &                    FAIL, ERRMSG )
C
C VERSION
C     13FEB15 AD Add Log message describing sub-sampling
C     12DEC13 AD Allow for RFMT as first data item in file
C     13NOV14 AD Allow for extra DV parameter in .tab header
C     16JAN14 AD Original. Extracted from SPCLUT.
C
C DESCRIPTION
C     Open TAB-LUT file and read in header info.
C     Called by LUTCHK,LUTFIL and SPCLUT for each LUT file.
C     This loads file header data into /TABCOM/ 
C     If JGAS=0 on input, JGAS is returned as the absorber index (or 0 if 
C       file absorber not required) and the file is closed.
C     If JGAS>0 in input a fatal error occurs unless the file absorber matches
C       JGAS, and the file is left open
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER       LUNLUT  !  I  LUN for TAB file
      CHARACTER*(*) NAMLUT  !  I  Name of TAB file
      LOGICAL       BINFIL  !  I  T=binary file, F=ASCII file
      INTEGER       JGAS    ! I/O Expected/found absorber index
      INTEGER       NDP     !  I  p-axis subsample factor (1=no subsampling)
      INTEGER       NDT     !  I  T-axis subsample factor (1=no subsampling)
      LOGICAL       FAIL    !  O  Set TRUE if a fatal error is detected
      CHARACTER*80  ERRMSG  !  O  Error message returned if FAIL is TRUE
C
      EXTERNAL
     &  NAMLOG ! Send text plus variable length filename to log file.
     &, IGASID ! Determine GASCOM index for given Gas#ID and Iso#ID
     &, OPNFIL ! Open ASCII input file, and skip/Log any initial comments.
     &, RFMLOG ! Write text message to RFM log file.
     &, SUBAXS ! Subsample (p,T) axes from LUT input file.
      INTEGER IGASID
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C COMMON VARIABLES
      INCLUDE 'tabcom.inc' ! Table axes for tabulated absorb.coeffs.
C
C LOCAL VARIABLES
      INTEGER      I       ! Counter for reading axis values
      INTEGER      IDGAS   ! HITRAN ID of gas, read from LUT file 
      INTEGER      IDISO   ! Isotopic ID of gas, read from LUT file
      INTEGER      IGAS    ! Index of gas in LUT file
      INTEGER      IOS     ! Saved value of IOSTAT for error messages
      INTEGER      NAMAX   ! Max size of any of p,T,q-axes
      LOGICAL      LCLOSE  ! T=close LUT file before return, F=keep open
      REAL         RGAS    ! Molec#ID plus Iso#ID read as a real number
      REAL         RFMT    ! Format identifier
      CHARACTER*5  GASSTR  ! Molec/Iso ID part of header record
      CHARACTER*80 LOGMSG  ! Message sent to log file
      CHARACTER*80 RECORD  ! Record read from LUT file
C
C EXECUTABLE CODE -------------------------------------------------------------
C
      IF ( BINFIL ) THEN
        CALL NAMLOG ( 'I-LUTTAB: Opening file:', NAMLUT, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
        OPEN ( UNIT=LUNLUT, FILE=NAMLUT, STATUS='OLD', 
     &         FORM='UNFORMATTED', IOSTAT=IOS, ERR=900 ) 
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) RECORD
        IF ( RECORD(1:1) .EQ. '!' ) THEN
          CALL RFMLOG ( RECORD, FAIL, ERRMSG )
          IF ( FAIL ) RETURN
        END IF
        DO WHILE ( RECORD(1:1) .EQ. '!' )
          READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) RECORD
        END DO
      ELSE
        CALL OPNFIL ( LUNLUT, NAMLUT, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
        READ ( LUNLUT, '(A)', IOSTAT=IOS, ERR=900 ) RECORD
      END IF     
C
C First record contains format identifier variable
      READ ( RECORD, * ) RFMT
      IF ( RFMT .NE. 1.0 ) THEN
        WRITE ( ERRMSG, '(A,F10.2)' ) 
     &    'F-LUTTAB: unrecognised LUT file format, version=', RFMT
        FAIL = .TRUE.
        RETURN
      END IF
C
C Next record contains Gas ID and array information
      IF ( BINFIL ) THEN
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) RECORD
      ELSE
        READ ( LUNLUT, '(A)', IOSTAT=IOS, ERR=900 ) RECORD
      END IF 
C      
      GASSTR = RECORD(1:5) 
      IF ( INDEX ( GASSTR, '.' ) .EQ. 0 ) THEN
        READ ( GASSTR, * ) IDGAS
        IDISO = -1
      ELSE
        READ ( GASSTR, * ) RGAS
        IDGAS = INT ( RGAS ) 
        IDISO = NINT( 10*(RGAS-IDGAS) )
      END IF            
C
      IGAS = IGASID ( IDGAS, IDISO )
      LCLOSE = JGAS .EQ. 0       ! If called from LUTFIL, close LUT file
      IF ( JGAS .EQ. 0 ) THEN
        IF ( IGAS .EQ. 0 ) THEN   ! not a reqd absorber
          GOTO 100                
        ELSE
          JGAS = IGAS
        END IF
C Shouldn't happen but possible that file has changed since initially checked
C while reading in driver table
      ELSE 
        IF ( IGAS .NE. JGAS ) STOP 'F-LUTTAB: Logical error#1'
      END IF
C
      READ ( RECORD(6:), * ) 
     &  NVTAB, V1TAB, V2TAB, DVTAB, NATAB, NPTAB, NTTAB, NQTAB
      OFFTAB = NTTAB .LT. 0
      NTTAB = ABS ( NTTAB )
C
      NAMAX = MAX ( NPTAB, NTTAB, NQTAB )
      IF ( NAMAX .GT. MAXTAB ) THEN
        FAIL = .TRUE.
        WRITE ( ERRMSG, '(A,I11,A,I11)' )
     &   'F-TABHDR: File contains axis size=', NAMAX,
     &   ' > MAXTAB in rfmsiz.inc, =', MAXTAB
        RETURN
      END IF 
C
      IF ( BINFIL ) THEN
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) (PAXTAB(I), I=1, NPTAB)
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) (TEMTAB(I), I=1, NPTAB)
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) (VMRTAB(I,1), I=1, NPTAB)
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) (TAXTAB(I), I=1, NTTAB)
        READ ( LUNLUT, IOSTAT=IOS, ERR=900 ) (QAXTAB(I), I=1, NQTAB)
      ELSE
        READ ( LUNLUT, *, IOSTAT=IOS, ERR=900 ) (PAXTAB(I), I=1, NPTAB)
        READ ( LUNLUT, *, IOSTAT=IOS, ERR=900 ) (TEMTAB(I), I=1, NPTAB)
        READ ( LUNLUT, *, IOSTAT=IOS, ERR=900 ) (VMRTAB(I,1), I=1,NPTAB)
        READ ( LUNLUT, *, IOSTAT=IOS, ERR=900 ) (TAXTAB(I), I=1, NTTAB)
        READ ( LUNLUT, *, IOSTAT=IOS, ERR=900 ) (QAXTAB(I), I=1, NQTAB)
      END IF
C
      IF ( NDP .NE. 1 .OR. NDT .NE. 1 ) THEN
        WRITE ( LOGMSG, '(A,2I11)' )
     &    'I-LUTTAB: Subsampling p,T axes by factors ', NDP, NDT
        CALL RFMLOG ( LOGMSG, FAIL, ERRMSG )
        IF ( FAIL ) RETURN
        CALL SUBAXS ( NDP, NDT )
      END IF
C
 100  CONTINUE
      IF ( LCLOSE ) CLOSE ( LUNLUT, IOSTAT=IOS, ERR=900 )
      FAIL = .FALSE.
      RETURN
C
  900 WRITE ( ERRMSG, '(A,I11)')
     &  'F-LUTTAB: I/O failure on LUT file. IOSTAT=', IOS
      FAIL = .TRUE.
C
      END
