      SUBROUTINE REACIA ( LUNCIA, WN1REQ, WN2REQ, MAXN,
     &                    EOF, ID1, ID2, TEM, N, M, WNO, CIA, 
     &                    FAIL, ERRMSG )
C
C VERSION
C     12JUL15 AD Original. 
C 
C DESCRIPTION    
C     Read Temperature tabulation from .cia file
C     Called multiply by CIAFIL.
C     Self-contained routine - no externals or included files.
C
C     A HITRAN .cia file is subdivided into tabulations of wavenumber and 
C     collisions-induced absorption for a particular pair of molecules,
C     spectral range and temperature.
C
C     For a given set of modelled CIA absorbers and required spectral range,
C     this procedure steps through a file until it finds tabulation for a 
C     modelled pair of molecules and overlapping spectral range.
C     The data matching the required spectral range are then extracted.
C     If the spectral range in the file is wider than the required range, an
C     extra point just below and/or above the required range are also extracted.
C
C     The user specifies the maximum available array size MAXN for WNO,CIA
C     N (LE MAXN) is the number of array elements in WNO,CIA actually written,
C     M is the number of array elements that would have been written if space
C       allowed.
C
C     If no (more) data are found, the subroutine returns EOF=.TRUE. and other
C     outputs (apart from FAIL, ERRMSG) are undefined.
C
      IMPLICIT NONE
C
C ARGUMENTS      
      INTEGER      LUNCIA     !  I  LUN for reading in data
      DOUBLE PRECISION WN1REQ !  I  Min Wavenumber [cm-1] reqd for RFM run
      DOUBLE PRECISION WN2REQ !  I  Max Wavenumber [cm-1] reqd for RFM run
      INTEGER      MAXN       !  I  Externally defined array dim for WNO,CIA
      LOGICAL      EOF        !  O  T=no more data in file, F=data read
      INTEGER      ID1        !  O  HITRAN/RFM index of first molecule
      INTEGER      ID2        !  O  HITRAN/RFM index of second molecule
      REAL         TEM        !  O  Temperature [K] of tabulation
      INTEGER      N          !  O  No. array elements WNO,CIA filled
      INTEGER      M          !  O  No. array elements WNO,CIA actually reqd
      DOUBLE PRECISION WNO(*) !  O  Waveno [cm-1] axis for CIA values
      REAL         CIA(*)     !  O  Coll.Induced Abs. coeffs [cm^5/kmole^2]
      LOGICAL      FAIL       !  O  T=a fatal error has been detected
      CHARACTER*80 ERRMSG     !  O  Error message written if FAIL is TRUE
C
      EXTERNAL
     &  CIAMOL ! Identify molecules from .cia file header record
C
C GLOBAL CONSTANTS
      INCLUDE 'phycon.inc' ! Physical and Mathematical constants
C
C LOCAL CONSTANTS
      DOUBLE PRECISION DAVOG2   ! square of avogadro's number ~6e26.
        PARAMETER ( DAVOG2 = DBLE ( AVOG )**2 )
C
C LOCAL VARIABLES
      INTEGER      IOS     ! Saved value of IOSTAT for error messages
      INTEGER      IPT     ! Counter for points within tabulation
      INTEGER      NPT     ! No. of data points in current tabulation
      LOGICAL      FINISH  ! T=reached end of required spectral range
      CHARACTER*20 MOLID   ! Part of .cia file header containing molecule names
      DOUBLE PRECISION DCIA ! Abs.coefficient [cm^5/molec^2] read from .cia file
      DOUBLE PRECISION W   ! Wavenumber axis value [cm-1] read from file
      DOUBLE PRECISION WN1 ! Lower wno limit [cm-1] of tabulation
      DOUBLE PRECISION WN2 ! Upper wno limit [cm-1] of tabulation
C
C EXECUTABLE CODE -------------------------------------------------------------
C
      FAIL = .FALSE.
      EOF = .FALSE.
C EOF=TRUE never actually occurs in loop - either exit subroutine with data or 
C jump to 800 when END=800 occurs
      DO WHILE ( .NOT. EOF )     
        READ ( LUNCIA, '(A20,2F10.4,I7,F7.1,F10.4)', IOSTAT=IOS,
     &         END=800, ERR=900 ) MOLID, WN1, WN2, NPT, TEM
C
C Skip data if outside required wavenumber range 
        IF ( WN1 .GT. WN2REQ .OR. WN2 .LT. WN1REQ ) THEN 
          DO IPT = 1, NPT
            READ ( LUNCIA, *, IOSTAT=IOS, ERR=900 ) W, DCIA
          END DO
C Overlaps required range so read in data
        ELSE 
          CALL CIAMOL ( MOLID, ID1, ID2, FAIL, ERRMSG ) 
          IF ( FAIL ) RETURN
          M = 0
          FINISH = .FALSE.
          DO IPT = 1, NPT
            READ ( LUNCIA, *, IOSTAT=IOS, ERR=900 ) W, DCIA
            IF ( .NOT. FINISH ) THEN
C Ensure that last point below start of required range is read in as array
C element#1
              IF ( W .LT. WN1REQ ) M = 0
              M = M + 1
              N = MIN ( M, MAXN ) 
              WNO(N) = W
              CIA(N) = MAX ( 0.0, SNGL ( DCIA * DAVOG2 ) )
              FINISH = W .GE. WN2REQ
            END IF
          END DO
          RETURN           ! Normal exit with new data loaded
        END IF
      END DO               ! read next tabulation
C
C Exit with end-of-file reached without loading (more) new data
  800 CONTINUE   
      EOF = .TRUE.
      FAIL = .FALSE.
      RETURN
C
  900 FAIL = .TRUE. 
      WRITE ( ERRMSG, '(A,I11)' ) 
     &  'F-REACIA: I/O error on CIA file, IOSTAT=', IOS
C
      END
