      SUBROUTINE REAXSC ( LUNXSC, IXSC, MAXXSP, MAXXST, 
     &                    EOF, IDGAS, WN1XSC, WN2XSC, NXSP, NXST, 
     &                    NPTXST, PREXST, TEMXST, WN1XST, WN2XST, 
     &                    ABSXSP, FAIL, ERRMSG )
C
C VERSION
C     02JUL15 AD Update LLUN if not same as LUNXSC
C     16FEB15 AD Allow for old format as well as HITRAN2K
C     05FEB15 AD Check HITRAN2K format after second attempt at reading.
C     04DEC14 AD Original. Extracted from rfmxsc.for
C 
C DESCRIPTION    
C     Read dataset from .xsc file
C     Called multiply by XSCFIL and RFMXSC 
C     Self-contained routine - no externals or included files.
C
C     A dataset within the .xsc file is defined by a particular molecule and
C     spectral range, starting with the RFM '***' header (as inserted by
C     program hitxsc.f (www.atm.ox.ac.uk/RFM/hitxsc.html). Within each dataset
C     there can be multiple tabulations for different (p,T) conditions with
C     nominally the same spectral range but different spectral grid spacings
C
C     This routine can be used with MAXXSP,MAXXST set to 1 if only the data
C     header info is required (molecule ID, spectral range, sizes).
C     If .xsc file contains more space than can be accommodated by declared
C     external array sizes, the array arguments will only be filled up to the
C     declared sizes but NXSP,NXST will be set to the required sizes and 
C     should be checked against MAXXSP, MAXXST externally.
C
C     Only supports the HITRAN2k format of x/s data as processed by program
C     hitxsc.f.
C                  
      IMPLICIT NONE
C
C ARGUMENTS      
      INTEGER   LUNXSC    !  I  LUN for reading in data
      INTEGER   IXSC      !  I  Index of molec/spc.range dataset in file
      INTEGER   MAXXSP    !  I  Size of ABSXSP array declared externally
      INTEGER   MAXXST    !  I  Size of *XST array args declared externally
      LOGICAL   EOF       !  O  T=no more data in file, F=data read
      INTEGER   IDGAS     !  O  RFM index of moecule
      DOUBLE PRECISION WN1XSC !  O  Min Wavenumber [cm-1] for dataset
      DOUBLE PRECISION WN2XSC !  O  Max Wavenumber [cm-1] for dataset
      INTEGER   NXSP      !  O  Tot. no. of tabulated abs.values in dataset
      INTEGER   NXST      !  O  No.of (p,T) tabulations in dataset
      INTEGER   NPTXST(*) !  O  No.of abs.coeff.values in each (p,T) set
      REAL      PREXST(*) !  O  List of dataset Pres.tab. pts [Torr]
      REAL      TEMXST(*) !  O  List of X/S data temp.tab. pts [K]
      DOUBLE PRECISION WN1XST(*) !  O  Min Wno [cm-1] for each (p,T) set      
      DOUBLE PRECISION WN2XST(*) !  O  Max Wno [cm-1] for each (p,T) set      
      REAL      ABSXSP(*) !  O  Absorption coefficient [cm2/molec]
      LOGICAL   FAIL      !  O  T=a fatal error has been detected
      CHARACTER*80 ERRMSG !  O  Error message written if FAIL is TRUE
C
      EXTERNAL
     &  UPCASE ! Convert text string to upper case.
C
C LOCAL VARIABLES
      INTEGER  IOS        ! Saved value of IOSTAT for error messages
      INTEGER  IXSP       ! Index for X/S abs.coeff. points
      INTEGER  IXST,JXST  ! Counter for X/S (p,T) tables
      INTEGER  LLUN       ! LUN used on previous call of this routine
      INTEGER  LXSC       ! Last value of IXSC argument for this routine
      INTEGER  NPT        ! No. of X/S data points in current (p,T) table
      LOGICAL  GOTRNG     ! T=got spectral range from RFM header, F=not there
      LOGICAL  H2KFMT     ! T=HITRAN 2000 format, F=Earlier format
      REAL     RDUMMY     ! Dummy real for skipping abs.coeff data
      CHARACTER*10  FMT   ! File Format identifier
      CHARACTER*100 RECORD ! Record read from X/S file
C
C DATA STATEMENTS
      DATA LLUN / 0 /
      DATA LXSC / 0 /
      SAVE LLUN, LXSC
C
C EXECUTABLE CODE -------------------------------------------------------------
C
C If this is a new LUN, then rewind to ensure pointer at the start of file
C otherwise assume that pointer at the end of dataset #LXSC
      IF ( LLUN .NE. LUNXSC .OR. LXSC .GE. IXSC ) THEN
        REWIND ( LUNXSC, IOSTAT=IOS, ERR=900 )
        LXSC = 0
        LLUN = LUNXSC
      END IF
      EOF = .TRUE.
      DO WHILE ( LXSC .LT. IXSC )
        READ ( LUNXSC, '(A)', IOSTAT=IOS, ERR=900, END=800 ) RECORD
        IF ( RECORD(1:3) .EQ. '***' ) LXSC = LXSC + 1
      END DO
      EOF = .FALSE.
C
C First see if wavenumber range is included in header (true for more recent
C files created by hitxsc but not older data )
      WN1XSC = 0.0D0
      WN2XSC = 0.0D0
      READ ( RECORD, '(3X,I7,I10,A10,2F10.4)', IOSTAT=IOS ) 
     &   IDGAS, NXST, FMT, WN1XSC, WN2XSC
      GOTRNG = IOS .EQ. 0 .AND. WN2XSC .NE. 0.0  ! got wno range from header
      IF ( .NOT. GOTRNG ) THEN
        READ ( RECORD, '(3X,I7,I10,A10)', IOSTAT=IOS, ERR=900 ) 
     &    IDGAS, NXST, FMT
        WN1XSC = 1.0D6
        WN2XSC = 0.0D0
      END IF
      CALL UPCASE ( FMT, FMT ) 
      H2KFMT = INDEX ( FMT, 'HITRAN2K' ) .GT. 0
C
C Loop over (p,T) subsets
      NXSP = 0
      DO JXST = 1, NXST
        IXST = MIN ( JXST, MAXXST )
        IF ( H2KFMT ) THEN
          READ ( LUNXSC, '(20X,2F10.4,I7,F7.2,F6.1)', 
     &           IOSTAT=IOS, ERR=900)
     &      WN1XST(IXST), WN2XST(IXST), NPTXST(IXST), TEMXST(IXST), 
     &      PREXST(IXST)
C If spectral range not in header, construct from individual pT table ranges
        ELSE
          READ ( LUNXSC, '(A)', IOSTAT=IOS, ERR=900 ) RECORD
          READ ( RECORD(11:60), *, IOSTAT=IOS, ERR=900 ) 
     &      WN1XST(IXST), WN2XST(IXST), NPTXST(IXST), TEMXST(IXST),
     &      PREXST(IXST) 
        END IF
        IF ( .NOT. GOTRNG ) THEN
          WN1XSC = MIN ( WN1XSC, WN1XST(IXST) )
          WN2XSC = MAX ( WN2XSC, WN2XST(IXST) )
        END IF
        NPT = NPTXST(IXST)
        IF ( NXSP + NPT .LE. MAXXSP ) THEN
          READ ( LUNXSC, '(10F10.3)', ERR=900, IOSTAT=IOS )
     &      ( ABSXSP(IXSP), IXSP = NXSP + 1, NXSP + NPT )
        ELSE
          READ ( LUNXSC, '(10F10.3)', ERR=900, IOSTAT=IOS )
     &      ( RDUMMY, IXSP = 1, NPT )
        END IF
        NXSP = NXSP + NPT
      END DO
C
  800 CONTINUE   ! jump here if end-of-file reached, with EOF=T
      FAIL = .FALSE.
      RETURN
C
  900 FAIL = .TRUE. 
      WRITE ( ERRMSG, '(A,I11)' ) 
     &  'F-REAXSC: I/O failure on X/S file, IOSTAT=', IOS
C
      END
