      SUBROUTINE RFMINP ( LUNDRV, LUNNXT, FAIL, ERRMSG )
C
C VERSION
C     02JUL15 AD Add INPCIA
C     20FEB14 AD Correction: call NXTREC after OPNFIL
C     12FEB14 AD Add INPPHY, remove INPCRV
C     12DEC13 AD Rewritten and simplified. Move most of checks on required or
C                repeated sections into new module KEYCHK
C     17OCT13 AD Move tests for duplicate sections here rather than in
C                INP*, also add SKPSEC  
C     14AUG13 AD Correction: add LUNNXT argument to INPSFC
C     19SEP12 AD Allow for records up to C*200 instead of C*80
C     13SEP11 AD Add INPRJT
C     05JUL07 AD Bug#65 Allow for improperly terminated driver tables
C     06JAN04 AD Add INPPRF
C     01SEP96 AD Version 1.
C
C DESCRIPTION
C     Open RFM Driver file and read input data.
C     Called once by RFM.
C
      IMPLICIT NONE
C
C ARGUMENTS
      INTEGER      LUNDRV  !  I  LUN for Driver File
      INTEGER      LUNNXT  ! I/O Next free LUN 
      LOGICAL      FAIL    !  O  Set TRUE if an error is detected
      CHARACTER*80 ERRMSG  !  O  Error message written if FAIL is TRUE
C
      EXTERNAL
     &  INPATM ! Read RFM atmospheric profiles from driver table 
     &, INPCHK ! Cross-check inputs from more than one drv table section
     &, INPCIA ! Read Collision-Induced Abs. filenames from RFM driver table.
     &, INPDIM ! Read TAB dims from driver table *TAN or *DIM section
     &, INPFIN ! Read user-defined fine-mesh resolution
     &, INPFLG ! Read RFM option flags from driver table
     &, INPFOV ! Read FOV data file from RFM driver table and load.
     &, INPGAS ! Read RFM absorbing gases from driver table 
     &, INPGRD ! Read list of GRD files of user-defined irregular grids
     &, INPHDR ! Read RFM Driver Header record from driver table
     &, INPHIT ! Read HITRAN line data filename from RFM driver table.
     &, INPILS ! Read list of ILS data files from driver table
     &, INPJAC ! Read details of Jacobians to be calculated
     &, INPLEV ! Read altitude levels for intermediate outputs.
     &, INPLUT ! Read list of Look-Up tables of absorption coefficients.
     &, INPNAM ! Read user-defined output filenames from RFM driver table.
     &, INPNTE ! Read non-LTE files from driver table 
     &, INPOBS ! Read user-defined observer position
C
        EXTERNAL
     &  INPPHY ! Read modified values of adjustable physical constants.
     &, INPREJ ! Read minimum line strength limits from RFM driver table.
     &, INPSFC ! Read RFM surface parameters from driver table 
     &, INPSHP ! Read RFM line shape model from driver table 
     &, INPSPC ! Read RFM wavenumber range and resolution from driver table 
     &, INPTAN ! Read RFM tangent points from driver table 
     &, INPTPS ! Read tabulated Total Internal Partition Sum data
     &, INPXSC ! Read RFM input Cross-section data files from driver table
     &, KEYCHK ! Check section headers read from RFM driver table
     &, NXTREC ! Load next record from RFM input file.
     &, OPNFIL ! Open ASCII input file, and skip/Log any initial comments.
     &, RFMLOG ! Write text message to RFM log file
     &, UPCASE ! Convert text string to upper case.
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmfil.inc' ! Standard filenames of RFM I/O files
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
C
C LOCAL CONSTANTS
      INTEGER       MINKEY        ! Minimum number of keyword labels required
        PARAMETER ( MINKEY = 6 )
C
C COMMON VARIABLES
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'ntecom.inc' ! Non-LTE data 
C
C LOCAL VARIABLES
      INTEGER       IKEY   ! Counter for mandatory keywords
      INTEGER       IOS    ! Saved value of IOSTAT for error messages
      LOGICAL       ENDSEC ! T = end of section found
      LOGICAL       GETHIT ! T = HITRAN data required (set in INPGAS)
      LOGICAL       GETNTE ! T = NTE data required
      LOGICAL       GETXSC ! T = X-Secn data required (set in INPGAS)  
      LOGICAL       IGNORE ! T = ignore section
      CHARACTER*200 RECORD ! Record read from Driver File
      CHARACTER*4   KEY    ! Part of driver table record tested for section key
      CHARACTER*80  WRNMSG ! Warning message for log file
C
C EXECUTABLE CODE -------------------------------------------------------------
C
C Open Driver file and print first record title to log file
      CALL OPNFIL ( LUNDRV, NAMDRV, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
C Skip last comment record (if any) before *HDR section
      CALL NXTREC ( LUNDRV, RECORD, ENDSEC, FAIL, ERRMSG )
      IF ( FAIL ) RETURN
C
C Read through first 6 Primary Sections (fixed order expected)
      DO IKEY = 1, 6
        READ ( LUNDRV, '(A)', IOSTAT=IOS, ERR=900 ) RECORD
        CALL RFMLOG ( RECORD, FAIL, ERRMSG )         ! Copy to LOG file
          IF ( FAIL ) RETURN
        CALL UPCASE ( RECORD(1:4), KEY )
        IF ( IKEY .EQ. 1 .AND. KEY .EQ. '*HDR' ) THEN              
          CALL INPHDR ( LUNDRV, FAIL, ERRMSG )            
        ELSE IF ( IKEY .EQ. 2 .AND. KEY .EQ. '*FLG' ) THEN
          CALL INPFLG ( LUNDRV, FAIL, ERRMSG )
        ELSE IF ( IKEY .EQ. 3 .AND. KEY .EQ. '*SPC' ) THEN
          CALL INPSPC ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( IKEY .EQ. 4 .AND. KEY .EQ. '*GAS' ) THEN
          CALL INPGAS ( LUNDRV, LUNNXT, GETHIT, GETXSC, FAIL, ERRMSG )
        ELSE IF ( IKEY .EQ. 5 .AND. KEY .EQ. '*ATM' ) THEN
          CALL INPATM ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( IKEY .EQ. 6 .AND. ( 
     &              KEY .EQ. '*TAN' .OR. KEY .EQ. '*GEO' .OR.
     &              KEY .EQ. '*ELE' .OR. KEY .EQ. '*SEC' .OR.
     &              KEY .EQ. '*LEV' .OR. KEY .EQ. '*LEN' .OR.
     &              KEY .EQ. '*DIM' ) ) THEN 
          IF ( TABFLG ) THEN
            CALL INPDIM ( LUNDRV, LUNNXT, KEY, FAIL, ERRMSG )
          ELSE           
            CALL INPTAN ( LUNDRV, LUNNXT, KEY, FAIL, ERRMSG )
          END IF
        ELSE 
          ERRMSG = 'F-RFMINP: Unexpected Primary Section in Drv.file: '
     &              //KEY
          FAIL = .TRUE.
        END IF
        IF ( FAIL ) RETURN
      END DO
C
C Check if NTE data section is required
      GETNTE = NTEFLG .AND. NNTE .EQ. 0 ! might have been loaded in .atm files
C
C Read through secondary sections, ignoring any which are not required
      DO WHILE ( KEY .NE. '*END' )
        READ ( LUNDRV, '(A)', IOSTAT=IOS, ERR=900 ) RECORD
        IF ( IOS .EQ. -1 ) THEN                       ! End of file
          WRNMSG = 'W-RFMINP: rfm.drv not properly terminated '//
     &      'with *END <CR> record - end assumed' 
          CALL RFMLOG ( WRNMSG, FAIL, ERRMSG )
          RECORD = '*END'
        ELSE
          CALL RFMLOG ( RECORD, FAIL, ERRMSG )
        END IF
        IF ( FAIL ) RETURN
        CALL UPCASE ( RECORD(1:4), KEY )
        CALL KEYCHK ( KEY, GETHIT, GETXSC, GETNTE, IGNORE, FAIL, ERRMSG)
          IF ( FAIL ) RETURN
        IF ( IGNORE ) THEN
          WRNMSG = 'W-RFMINP: Ignoring '//KEY//
     &         ' section - relevant option flag not enabled'
          CALL RFMLOG ( WRNMSG, FAIL, ERRMSG )
          ENDSEC = .FALSE.
          DO WHILE ( .NOT. ENDSEC )
            CALL NXTREC ( LUNDRV, RECORD, ENDSEC, FAIL, ERRMSG )
            IF ( FAIL ) RETURN
          END DO
        ELSE IF ( KEY .EQ. '*ABS' ) THEN
          CALL INPNAM ( LUNDRV, 'ABS', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*BBT' ) THEN
          CALL INPNAM ( LUNDRV, 'BBT', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*CIA' ) THEN
          CALL INPCIA ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*COO' ) THEN
          CALL INPNAM ( LUNDRV, 'COO', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*FIN' ) THEN
          CALL INPFIN ( LUNDRV, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*FOV' ) THEN
          CALL INPFOV ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*GRD' ) THEN
          CALL INPGRD ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*HIT' ) THEN
          CALL INPHIT ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*ILS' ) THEN
          CALL INPILS ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*JAC' ) THEN
          CALL INPJAC ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*LEV' ) THEN
          CALL INPLEV ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*LUT' ) THEN
          CALL INPLUT ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*NTE' ) THEN
          CALL INPNTE ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*OBS' ) THEN
          CALL INPOBS ( LUNDRV, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*OPT' ) THEN
          CALL INPNAM ( LUNDRV, 'OPT', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*PHY' ) THEN
          CALL INPPHY ( LUNDRV, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*PRF' ) THEN
          CALL INPNAM ( LUNDRV, 'PRF', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*PTH' ) THEN
          CALL INPNAM ( LUNDRV, 'PTH', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*RAD' ) THEN
          CALL INPNAM ( LUNDRV, 'RAD', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*REJ' ) THEN
          CALL INPREJ ( LUNDRV, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*RJT' ) THEN
          CALL INPNAM ( LUNDRV, 'RJT', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*SFC' ) THEN
          CALL INPSFC ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*SHP' ) THEN
          CALL INPSHP ( LUNDRV, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*TAB' ) THEN
          CALL INPNAM ( LUNDRV, 'TAB', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*TPS' ) THEN
          CALL INPTPS ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*TRA' ) THEN
          CALL INPNAM ( LUNDRV, 'TRA', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*WID' ) THEN
          CALL INPNAM ( LUNDRV, 'WID', FAIL, ERRMSG )
        ELSE IF ( KEY .EQ. '*XSC' ) THEN
          CALL INPXSC ( LUNDRV, LUNNXT, FAIL, ERRMSG )
        END IF
        IF ( FAIL ) RETURN
      END DO
C
C Normal exit
      CLOSE ( LUNDRV, IOSTAT=IOS, ERR=900 )
      CALL INPCHK ( FAIL, ERRMSG )    ! Sets FAIL=FALSE for exit if all OK
      RETURN
C
C Exit with Fatal I/O Error 
C 
  900 WRITE ( ERRMSG, '(A,I11)' ) 
     &  'F-RFMINP: I/O Failure on Driver File. IOSTAT=', IOS
      FAIL = .TRUE.
C
      END
