      SUBROUTINE RFMREX 
C
C VERSION
C     01JAN15 AD Original.
C 
C DESCRIPTION    
C     Calculate the contribution of Rayleigh extinction.
C     Called by RFM for each wide mesh interval if REX flag enabled.
C
C REFERENCES
C     Bodhaine, B. A., N. B. Wood, E. G. Dutton, and J. R. Slusser,
C       On Rayleigh Optical Depth Calculations, J. Atm. Ocean. Tech.,
C       16(11), 1854--1861, 1999.
C     Eq(2) gives scattering cross-section per molecule
C       sigma = [24 pi^3 (n^2-1)^2]/[lambda^4 rho^2 (n^2+2)^2] * K
C     where n is refractive index, lambda = wavelength, rho=number density,
C     K is the King Factor, or depolarization term.
C     Note that (n^2-1)^2 is effectively units of molecules
C     Writing n = N+1, where N is the refractivity <<1, this is approximately
C       sigma = [24 pi^3 (2N)^2]/[lambda^4 rho^2 3^2] * K
C             = [32 pi^3/3] ( N^2/rho^2 ) (1/lambda^4) * K
C
C      Refractivity depends primarily on density, so N/rho is virtually constant
C      with altitude, so it should be OK to use value at lower boundary of layer
C      rather than anything more detailed.
C      Bodhaine expression N/rho is implicitly for 1 molecule, so square has 
C      dimension [cm^6/molec] rather than [cm^6/molec^2]
C      Similarly for King factor, which depends on air mixture therefore varies
C      only slowly with altitude
C               
      IMPLICIT NONE
C
      EXTERNAL
     &  KINGDF ! Calculate the King Factor for depolarization
      REAL KINGDF
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
      INCLUDE 'phycon.inc' ! Physical and Mathematical constants
      INCLUDE 'shpcon.inc' ! Line-shape codes
C
C LOCAL CONSTANTS
      REAL PIFACT             ! Numerical factor
        PARAMETER ( PIFACT = 32.0 * PI**3 / 3.0 )
C
C COMMON VARIABLES
      INCLUDE 'atmcom.inc' ! Atmospheric profile data 
      INCLUDE 'fincom.inc' ! Fine mesh data
      INCLUDE 'gascom.inc' ! Molecular and isotope data
      INCLUDE 'pthcom.inc' ! RFM path data
C
C LOCAL VARIABLES
      INTEGER IATM   ! Index of atmospheric layer
      INTEGER IFIN   ! Counter for fine grid points
      INTEGER IGAS   ! Index of absorber
      INTEGER IPTH   ! Counter for path elements
      REAL    RFRDEN ! Squared Ratio of refractivity/density
      REAL    SIGMA  ! Spectrally constant part of extinction 
      DOUBLE PRECISION WNO ! Wavenumber [cm-1]
C
C EXECUTABLE CODE -------------------------------------------------------------
C
      DO IPTH = 1, NCLC
        IGAS = IGSPTH(IPTH)
        IF ( SHPGAS(IGAS) .EQ. SHPAIR ) THEN
          IATM = IATPTH(IPTH)
C DNSATM is in units of molec/cm^3, so scale by AVOG to avoid RFRDEN=0 when
C taking square
          RFRDEN = (AVOG*RFRATM(IATM)/DNSATM(IATM))**2 ! cm^6/molec * AVOG^2
          SIGMA = PIFACT * RFRDEN * AMTPTH(IPTH) / AVOG ! cm^4 
          DO IFIN = 1, NFIN
            WNO = WNOFIN(IFIN) 
            ABSFIN(IFIN,IPTH) = SIGMA * WNO**4 * KINGDF(WNO,IATM)
            CNTFIN(IFIN,IPTH) = ABSFIN(IFIN,IPTH)
          END DO
        END IF
      END DO
C
      END
