      SUBROUTINE RFMXSC ( FAIL, ERRMSG )
C
C VERSION
C     03AUG15 AD Ensure XSC >= 0 before accumulating
C     30JUN15 AD Add LNPINT to allow interpolation in lnp rather than p
C     21NOV14 AD Use REAXSC to load dataset from .xsc file, 
C                Add XSCBUF to allow more than one XSC dataset in memory
C                Remove xflcom.inc
C     03SEP14 AD Bug#93: assigning 102 as N2O5 for V42 flag.
C     08AUG13 AD New molecule indices - check for old values still in use
C                    No temperature extrapolation unless V42 flag enabled.
C     01DEC10 AD Keep separate MIN/MAX WNO for each table.
C     01SEP96 AD Version 1.
C 
C DESCRIPTION    
C     Calculate the absorption due to cross-section species 
C     Called by RFM for each wide mesh interval.
C     This new version attempts to fit multiple X/S datasets into the *XST,*XSP
C     arrays, deleting space as the datasets are no longer required. This is to
C     avoid the slow process of reloading the data each time from the .xsc files
C     when two or more datasets cover the same spectral range.
C                  
      IMPLICIT NONE
      SAVE
C
C ARGUMENTS      
      LOGICAL      FAIL    !  O  T=a fatal error has been detected
      CHARACTER*80 ERRMSG  !  O  Error message written if FAIL is TRUE
C
      EXTERNAL
     &  IDGNEW ! Convert old RFM index for .xsc data to new value
     &, QTINT  ! Calculate total internal partition sums for tabulated data.   
     &, REAXSC ! Read dataset from .xsc file
     &, TRIANG ! Interpolation of irregular 2D grid by triangulation.
     &, XSCBUF ! Handle local memory storage of multiple X/S datasets
      INTEGER IDGNEW
C
C GLOBAL CONSTANTS
      INCLUDE 'rfmsiz.inc' ! RFM Array sizes
      INCLUDE 'phycon.inc' ! Physical and Mathematical constants
      INCLUDE 'shpcon.inc' ! Line-shape codes
C
C COMMON VARIABLES
      INCLUDE 'fincom.inc' ! Fine mesh data
      INCLUDE 'flgcom.inc' ! RFM option flags
      INCLUDE 'gascom.inc' ! Molecular and isotope data
      INCLUDE 'idxcon.inc' ! HITRAN molecular indices
      INCLUDE 'pthcom.inc' ! RFM path data
      INCLUDE 'xsccom.inc' ! X-section data 
C
C LOCAL CONSTANTS
      LOGICAL LNPINT       ! F=interpolate linearly in p, T= in lnp
        PARAMETER ( LNPINT = .FALSE. ) 
      REAL TEMTOL          ! Temp. tolerance [K] before applying QTINT factor
        PARAMETER ( TEMTOL = 0.1 ) 
C
C LOCAL VARIABLES
      integer i
      INTEGER IDGOLD       ! Old RFM index for some gases (for V42FLG)
      INTEGER IDXTRI(3,MAXCLC) ! Indices for triangulation points 
      INTEGER IFIN         ! Fine mesh grid point counter (0:NFIN)
      INTEGER IGAS         ! Gas counter
      INTEGER IOFXST(MAXXST) ! Offset index for each (p,T) dataset
      INTEGER IP1          ! Starting index for x/s dataset within *XSP arrays
      INTEGER IP           ! Counters for points within *XSP arrays
      INTEGER IPTH         ! Path counter
      INTEGER IT           ! Counter for points within *XST arrays
      INTEGER IT1          ! Starting index for x/s dataset within *XST arrays
      INTEGER IXSC         ! Counter for X/S data sets
      INTEGER JP           ! IP+1, for interpolation within *XSP arrays
      INTEGER MOL          ! X/S Molecule ID code (51:63), 50=aerosol  
      INTEGER MP           ! Length *XSP arrays currently filled
      INTEGER MT           ! Length *XST arrays currently filled
      INTEGER NP           ! Size of x/s dataset within *XSP arrays
      INTEGER NPTXST(MAXXST) ! No. of abs.coeff points in (p,T) subset 
      INTEGER NT           ! Size of x/s dataset within *XST arrays
      LOGICAL EOF          ! T=end-of-file reached, F=not yet
      LOGICAL INIT         ! T=initialise triangulation routine
      LOGICAL RELOAD       ! T=(Re)load X/S dataset from file, F=already loaded
      LOGICAL REMOVE       ! T=remove X/S dataset from arrays, F=retain/ignore
      LOGICAL USEXSC       ! T=use this X/S dataset, F=don't use
      REAL    AMT          ! Gas amount in current path [ moles ? ]
      REAL    DP           ! Frac. of Waveno.tabul.intvl above FM Grid Pt.
      REAL    PRELIM       ! Interpolated pressre [Torr] from table
      REAL    PREXST(MAXXST) ! List of X/S data Pres.tab. pts [Torr]
      REAL    PTORR        ! Pressure [Torr] (760 Torr = 1 atm)
      REAL    SQ           ! Ratio of Tot.Partit.Sum at 296K to TPS at path tem
      REAL    SQLIM        ! Ratio of TIPS at 296K to TIPS at tabulation limit
      REAL    TEM          ! Temperature of current path
      REAL    TEMFAC(MAXCLC) ! Temperature extrapolation factor
      REAL    TEMLIM       ! Interpolated temperature from table
      REAL    TEMXST(MAXXST)   ! List of X/S data temp.tab. pts [K]
      REAL    WGTTRI(3,MAXCLC) ! Triangulation weights for each path
      REAL    XP            ! Position of FM Grid pt in X/S wno.tabulation axis
      REAL    XSC          ! Value of interpolated X/S data to path,FM Grid pt.
      REAL    XSCFIN(MAXXST) ! X/S data tabulation interp.to FM grid.
      REAL    XSCTAB(MAXXSP) ! X/S data tabulation
      DOUBLE PRECISION DDUMMY ! Dummy DP variable
      DOUBLE PRECISION DELXST(MAXXST) ! Wno spacing [cm-1] for each (p,T) subset
      DOUBLE PRECISION WN1XST(MAXXST) ! Min.Wno [cm-1] for each (p,T) subset
      DOUBLE PRECISION WN2XST(MAXXST) ! Max.Wno [cm-1] for each (p,T) subset
      DOUBLE PRECISION WNO ! Waveno of current Fine Mesh grid point
C
C EXECUTABLE CODE -------------------------------------------------------------
C
C Loop over cross-section data file
      DO IXSC = 1, NXSC
        IGAS = IGSXSC(IXSC)
        USEXSC = SHPGAS(IGAS) .EQ. SHPXSC .AND.          ! Not replaced by LUT
     &           WN2XSC(IXSC) .GE. WN1FIN .AND.
     &           WN1XSC(IXSC) .LE. WN2FIN 
        CALL XSCBUF ( USEXSC, IXSC, IT1, IP1, NT, NP, MT, MP, RELOAD,
     &                REMOVE ) 
C If XSC dataset no longer required (ie beyond upper wno limit) then move
C everything in memory forward to free up more space - this is quicker than 
C running out of space and having  to reload data from the .xsc file
        IF ( REMOVE ) THEN
          DO IT = IT1, MT 
            IOFXST(IT) = IOFXST(IT+NT)
            NPTXST(IT) = NPTXST(IT+NT)
            PREXST(IT) = PREXST(IT+NT)
            TEMXST(IT) = TEMXST(IT+NT)
            DELXST(IT) = DELXST(IT+NT)
            WN1XST(IT) = WN1XST(IT+NT)
            WN2XST(IT) = WN2XST(IT+NT)
          END DO
          DO IP = IP1, MP
            XSCTAB(IP) = XSCTAB(IP+NP)
          END DO
        ELSE IF ( USEXSC ) THEN 
          IF ( RELOAD ) THEN
            CALL REAXSC ( LUNXSC(IXSC), IDSXSC(IXSC), MAXXSP, MAXXST, 
     &                    EOF, MOL, DDUMMY, DDUMMY, NPXSC(IXSC), 
     &                    NTXSC(IXSC), NPTXST(IT1), PREXST(IT1), 
     &                    TEMXST(IT1), WN1XST(IT1), WN2XST(IT1), 
     &                    XSCTAB(IP1), FAIL, ERRMSG ) 
              IF ( FAIL ) RETURN
              IF ( EOF ) STOP 'F-RFMXSC: Logical error#1' 
            IF ( LNPINT ) THEN
              DO IT = IT1, IT1 + NTXSC(IXSC) - 1
                IF ( PREXST(IT) .GT. 0.0 ) 
     &            PREXST(IT) = 100 * LOG(PREXST(IT))
              END DO
            END IF
C Convert old xsc molecular index to new, if required
            MOL = IDGNEW ( MOL ) 
C For V42FLG also require old molecule ID for QTINT for 5 specific molecules
            IF ( V42FLG ) THEN
              IDGOLD = MOL
              IF ( MOL .EQ. 111 ) IDGOLD = 51    ! f11
              IF ( MOL .EQ. 122 ) IDGOLD = 56    ! f22
              IF ( MOL .EQ. 104 ) IDGOLD = 60    ! ccl4
              IF ( MOL .EQ. 101 ) IDGOLD = 61    ! clono2
              IF ( MOL .EQ. 102 ) IDGOLD = 62    ! n2o5   fixed Bug#93
            END IF
            IF (MOL .NE. IDXGAS(IGAS)) STOP 'F-RFMXSC: Logical Error#2'
            IP = IP1 - 1
            DO IT = IT1, IT1 + NT - 1
              DELXST(IT) = ( WN2XST(IT) - WN1XST(IT) ) / 
     &                       ( NPTXST(IT) - 1 ) 
              IOFXST(IT) = IP
              IP = IP + NPTXST(IT)
            END DO
            INIT = .TRUE.                             ! Set to FALSE by TRIANG
            DO IPTH = 1, NCLC  
              IF ( IGSPTH(IPTH) .EQ. IGAS ) THEN
                TEM = TEMPTH(IPTH)
C NB: Triangulation assumes temperature and pressure axes are appropriately
C scaled, so keep P in Torr so variation in each axis is a few hundred units.
                PTORR = PREPTH(IPTH) * 760.0          ! Convert atm to Torr
                IF ( LNPINT ) PTORR = 100*LOG ( PTORR ) 
                CALL TRIANG ( TEM, PTORR, NTXSC(IXSC), 
     &                        TEMXST(IT1), PREXST(IT1), INIT,
     &                        IDXTRI(1,IPTH), WGTTRI(1,IPTH),  
     &                        TEMLIM, PRELIM, FAIL, ERRMSG  )
                  IF ( FAIL ) RETURN 
C For old (v4.2) only ..
C If actual temperature (TEM) is beyond tabulation scale (TEMLIM), 
C extrapolate by applying partition function ratio SQ appropriate for each 
C molecule (not for aerosol, though)
                IF ( V42FLG .AND.             
     &               MOL .NE. IDXAER .AND.       
     &               ABS ( TEM - TEMLIM ) .GT. TEMTOL ) THEN
                  CALL QTINT ( IDGOLD, TEMLIM, SQLIM )
                  CALL QTINT ( IDGOLD, TEM,    SQ    )
                  TEMFAC(IPTH) = SQ / SQLIM
                ELSE                     ! no T extrapolation in new RFM
                  TEMFAC(IPTH) = 1.0
                END IF
              END IF
            END DO
          END IF           ! end RELOAD
C
C Interpolate the cross-section data onto the fine mesh
          IXSFIN(IGAS) = 0         ! info for WID diagnostics
          DO IFIN = 1, NFIN
            WNO = WNOFIN(IFIN) 
            I = 0
            DO IT = IT1, IT1 + NT - 1
              I = I + 1
              IF ( WNO .LT. WN1XST(IT) .OR. 
     &             WNO .GE. WN2XST(IT)      ) THEN
                XSCFIN(I) = 0.0
              ELSE
                IXSFIN(IGAS) = 1
                XP = 1.0 + SNGL ( ( WNO - WN1XST(IT) ) 
     &                                      / DELXST(IT) )
                IP = INT ( XP ) + IOFXST(IT)
                JP = MIN ( 1 + IP, MAXXSP )  ! limit index if WNO=WN2XST
                DP = MOD ( XP, 1.0 )
                XSCFIN(I) = XSCTAB(IP) * ( 1.0 - DP ) + 
     &                      XSCTAB(JP) * DP
              END IF
            END DO
C 
            DO IPTH = 1, NCLC                                 ! Loop over paths
              IF ( IGAS .EQ. IGSPTH(IPTH) ) THEN
                AMT = AVOG * AMTPTH(IPTH)
C
C Interpolate between temperature data sets, calculate absorption
                XSC = WGTTRI(1,IPTH) * XSCFIN(IDXTRI(1,IPTH)) +
     &                WGTTRI(2,IPTH) * XSCFIN(IDXTRI(2,IPTH)) +
     &                WGTTRI(3,IPTH) * XSCFIN(IDXTRI(3,IPTH))
                XSC = MAX ( XSC, 0.0 )  ! tabulated values could be slightly -ve
                ABSFIN(IFIN,IPTH) = ABSFIN(IFIN,IPTH) + 
     &                              XSC * AMT * TEMFAC(IPTH)
                CNTFIN(IFIN,IPTH) = ABSFIN(IFIN,IPTH)
              END IF
            END DO
          END DO
        END IF     ! end USEXSC
      END DO
C 
      FAIL = .FALSE.
C
      END
